@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 #include "qv4arraydata_p.h"
 #include "qv4object_p.h"
 #include "qv4functionobject_p.h"
@@ -43,33 +7,15 @@
 #include "qv4runtime_p.h"
 #include "qv4argumentsobject_p.h"
 #include "qv4string_p.h"
+#include "qv4jscall_p.h"
 
 using namespace QV4;
 
-QT_WARNING_SUPPRESS_GCC_TAUTOLOGICAL_COMPARE_ON
-
-const QV4::VTable QV4::ArrayData::static_vtbl = {
-    0,
-    0,
-    0,
-    0,
-    QV4::ArrayData::IsExecutionContext,
-    QV4::ArrayData::IsString,
-    QV4::ArrayData::IsObject,
-    QV4::ArrayData::IsFunctionObject,
-    QV4::ArrayData::IsErrorObject,
-    QV4::ArrayData::IsArrayData,
-    0,
-    QV4::ArrayData::MyType,
-    "ArrayData",
-    Q_VTABLE_FUNCTION(QV4::ArrayData, destroy),
-    0,
-    isEqualTo
-};
+DEFINE_MANAGED_VTABLE(ArrayData);
 
 const ArrayVTable SimpleArrayData::static_vtbl =
 {
-    DEFINE_MANAGED_VTABLE_INT(SimpleArrayData, 0),
+    DEFINE_MANAGED_VTABLE_INT(SimpleArrayData, nullptr),
     Heap::ArrayData::Simple,
     SimpleArrayData::reallocate,
     SimpleArrayData::get,
@@ -85,7 +31,7 @@ const ArrayVTable SimpleArrayData::static_vtbl =
 
 const ArrayVTable SparseArrayData::static_vtbl =
 {
-    DEFINE_MANAGED_VTABLE_INT(SparseArrayData, 0),
+    DEFINE_MANAGED_VTABLE_INT(SparseArrayData, nullptr),
     Heap::ArrayData::Sparse,
     SparseArrayData::reallocate,
     SparseArrayData::get,
@@ -99,18 +45,16 @@ const ArrayVTable SparseArrayData::static_vtbl =
     SparseArrayData::length
 };
 
-QT_WARNING_SUPPRESS_GCC_TAUTOLOGICAL_COMPARE_OFF
-
 Q_STATIC_ASSERT(sizeof(Heap::ArrayData) == sizeof(Heap::SimpleArrayData));
 Q_STATIC_ASSERT(sizeof(Heap::ArrayData) == sizeof(Heap::SparseArrayData));
 
-static Q_ALWAYS_INLINE void storeValue(ReturnedValue *target, uint value)
+void Heap::ArrayData::markObjects(Heap::Base *base, MarkStack *stack)
 {
-    Value v;
-    v.setEmpty(value);
-    *target = v.asReturnedValue();
+    ArrayData *a = static_cast<ArrayData *>(base);
+    a->values.mark(stack);
 }
 
+
 void ArrayData::realloc(Object *o, Type newType, uint requested, bool enforceAttributes)
 {
     Scope scope(o->engine());
@@ -138,8 +82,6 @@ void ArrayData::realloc(Object *o, Type newType, uint requested, bool enforceAtt
         if (d->type() > newType)
             newType = d->type();
     }
-    if (enforceAttributes && newType == Heap::ArrayData::Simple)
-        newType = Heap::ArrayData::Complex;
 
     while (alloc < requested)
         alloc *= 2;
@@ -161,7 +103,7 @@ void ArrayData::realloc(Object *o, Type newType, uint requested, bool enforceAtt
     }
     newData->setAlloc(alloc);
     newData->setType(newType);
-    newData->setAttrs(enforceAttributes ? reinterpret_cast<PropertyAttributes *>(newData->d()->values.values + alloc) : 0);
+    newData->setAttrs(enforceAttributes ? reinterpret_cast<PropertyAttributes *>(newData->d()->values.values + alloc) : nullptr);
     o->setArrayData(newData);
 
     if (d) {
@@ -188,39 +130,38 @@ void ArrayData::realloc(Object *o, Type newType, uint requested, bool enforceAtt
 
     Heap::SparseArrayData *sparse = static_cast<Heap::SparseArrayData *>(newData->d());
 
-    ReturnedValue *lastFree;
+    Value *lastFree;
     if (d && d->type() == Heap::ArrayData::Sparse) {
         Heap::SparseArrayData *old = static_cast<Heap::SparseArrayData *>(d->d());
         sparse->sparse = old->sparse;
-        old->sparse = 0;
-        sparse->freeList = old->freeList;
-        lastFree = &sparse->freeList;
+        old->sparse = nullptr;
+        lastFree = &sparse->sparse->freeList;
     } else {
         sparse->sparse = new SparseArray;
-        lastFree = &sparse->freeList;
-        storeValue(lastFree, 0);
+        lastFree = &sparse->sparse->freeList;
+        *lastFree = Encode(0);
         for (uint i = 0; i < toCopy; ++i) {
             if (!sparse->values[i].isEmpty()) {
                 SparseArrayNode *n = sparse->sparse->insert(i);
                 n->value = i;
             } else {
-                storeValue(lastFree, i);
+                *lastFree = Encode(i);
                 sparse->values.values[i].setEmpty();
-                lastFree = &sparse->values.values[i].rawValueRef();
+                lastFree = &sparse->values.values[i];
             }
         }
     }
 
     if (toCopy < sparse->values.alloc) {
         for (uint i = toCopy; i < sparse->values.alloc; ++i) {
-            storeValue(lastFree, i);
+            *lastFree = Encode(i);
             sparse->values.values[i].setEmpty();
-            lastFree = &sparse->values.values[i].rawValueRef();
+            lastFree = &sparse->values.values[i];
         }
-        storeValue(lastFree, UINT_MAX);
     }
+    *lastFree = Encode(-1);
 
-    Q_ASSERT(Value::fromReturnedValue(sparse->freeList).isEmpty());
+    Q_ASSERT(sparse->sparse->freeList.isInteger());
     // ### Could explicitly free the old data
 }
 
@@ -242,7 +183,7 @@ ReturnedValue SimpleArrayData::get(const Heap::ArrayData *d, uint index)
 {
     const Heap::SimpleArrayData *dd = static_cast<const Heap::SimpleArrayData *>(d);
     if (index >= dd->values.size)
-        return Primitive::emptyValue().asReturnedValue();
+        return Value::emptyValue().asReturnedValue();
     return dd->data(index).asReturnedValue();
 }
 
@@ -267,7 +208,7 @@ bool SimpleArrayData::del(Object *o, uint index)
         return true;
 
     if (!dd->attrs || dd->attrs[index].isConfigurable()) {
-        dd->setData(o->engine(), index, Primitive::emptyValue());
+        dd->setData(o->engine(), index, Value::emptyValue());
         if (dd->attrs)
             dd->attrs[index] = Attr_Data;
         return true;
@@ -323,14 +264,19 @@ uint SimpleArrayData::truncate(Object *o, uint newLen)
         return newLen;
 
     if (!dd->attrs) {
+        for (uint i = newLen; i < dd->values.size; ++i)
+            dd->setData(dd->internalClass->engine, i, Value::emptyValue());
         dd->values.size = newLen;
         return newLen;
     }
 
     while (dd->values.size > newLen) {
-        if (!dd->data(dd->values.size - 1).isEmpty() && !dd->attrs[dd->values.size - 1].isConfigurable())
+        const uint lastIndex = dd->values.size - 1;
+        if (!dd->data(lastIndex).isEmpty() && !dd->attrs[lastIndex].isConfigurable())
             return dd->values.size;
-        --dd->values.size;
+
+        dd->setData(dd->internalClass->engine, lastIndex, Value::emptyValue());
+        dd->values.size = lastIndex;
     }
     return dd->values.size;
 }
@@ -349,7 +295,7 @@ bool SimpleArrayData::putArray(Object *o, uint index, const Value *values, uint
     }
     QV4::ExecutionEngine *e = o->engine();
     for (uint i = dd->values.size; i < index; ++i)
-        dd->setData(e, i, Primitive::emptyValue());
+        dd->setData(e, i, Value::emptyValue());
     for (uint i = 0; i < n; ++i)
         dd->setData(e, index + i, values[i]);
     dd->values.size = qMax(dd->values.size, index + n);
@@ -362,12 +308,12 @@ void SparseArrayData::free(Heap::ArrayData *d, uint idx)
     Value *v = d->values.values + idx;
     if (d->attrs && d->attrs[idx].isAccessor()) {
         // double slot, free both. Order is important, so we have a double slot for allocation again afterwards.
-        v[1].setEmpty(Value::fromReturnedValue(d->freeList).emptyValue());
-        v[0].setEmpty(idx + 1);
+        v[1] = d->sparse->freeList;
+        v[0] = Encode(idx + 1);
     } else {
-        v->setEmpty(Value::fromReturnedValue(d->freeList).emptyValue());
+        *v = d->sparse->freeList;
     }
-    d->freeList = Primitive::emptyValue(idx).asReturnedValue();
+    d->sparse->freeList = Encode(idx);
     if (d->attrs)
         d->attrs[idx].clear();
 }
@@ -384,36 +330,34 @@ uint SparseArrayData::allocate(Object *o, bool doubleSlot)
     Q_ASSERT(o->d()->arrayData->type == Heap::ArrayData::Sparse);
     Heap::SimpleArrayData *dd = o->d()->arrayData.cast<Heap::SimpleArrayData>();
     if (doubleSlot) {
-        ReturnedValue *last = &dd->freeList;
+        Value *last = &dd->sparse->freeList;
         while (1) {
-            if (Value::fromReturnedValue(*last).value() == UINT_MAX) {
+            if (last->int_32() == -1) {
                 reallocate(o, dd->values.alloc + 2, true);
                 dd = o->d()->arrayData.cast<Heap::SimpleArrayData>();
-                last = &dd->freeList;
-                Q_ASSERT(Value::fromReturnedValue(*last).value() != UINT_MAX);
+                last = &dd->sparse->freeList;
+                Q_ASSERT(last->int_32() != -1);
             }
 
-            Q_ASSERT(dd->values[Value::fromReturnedValue(*last).value()].value() != Value::fromReturnedValue(*last).value());
-            if (dd->values[Value::fromReturnedValue(*last).value()].value() == (Value::fromReturnedValue(*last).value() + 1)) {
+            Q_ASSERT(dd->values[static_cast<uint>(last->int_32())].int_32() != last->int_32());
+            if (dd->values[static_cast<uint>(last->int_32())].int_32() == last->int_32() + 1) {
                 // found two slots in a row
-                uint idx = Value::fromReturnedValue(*last).emptyValue();
-                Value lastV = Value::fromReturnedValue(*last);
-                lastV.setEmpty(dd->values[lastV.emptyValue() + 1].value());
-                *last = lastV.rawValue();
+                uint idx = static_cast<uint>(last->int_32());
+                *last = Encode(dd->values[static_cast<uint>(last->int_32()) + 1].int_32());
                 dd->attrs[idx] = Attr_Accessor;
                 return idx;
             }
-            last = &dd->values.values[Value::fromReturnedValue(*last).value()].rawValueRef();
+            last = &dd->values.values[last->int_32()];
         }
     } else {
-        if (Value::fromReturnedValue(dd->freeList).value() == UINT_MAX) {
+        if (dd->sparse->freeList.int_32() == -1) {
             reallocate(o, dd->values.alloc + 1, false);
             dd = o->d()->arrayData.cast<Heap::SimpleArrayData>();
         }
-        uint idx = Value::fromReturnedValue(dd->freeList).value();
-        Q_ASSERT(idx != UINT_MAX);
-        dd->freeList = dd->values[idx].asReturnedValue();
-        Q_ASSERT(Value::fromReturnedValue(dd->freeList).isEmpty());
+        Q_ASSERT(dd->sparse->freeList.int_32() != -1);
+        uint idx = static_cast<uint>(dd->sparse->freeList.int_32());
+        dd->sparse->freeList = dd->values[idx];
+        Q_ASSERT(dd->sparse->freeList.isInteger());
         if (dd->attrs)
             dd->attrs[idx] = Attr_Data;
         return idx;
@@ -425,7 +369,7 @@ ReturnedValue SparseArrayData::get(const Heap::ArrayData *d, uint index)
     const Heap::SparseArrayData *s = static_cast<const Heap::SparseArrayData *>(d);
     index = s->mappedIndex(index);
     if (index == UINT_MAX)
-        return Primitive::emptyValue().asReturnedValue();
+        return Value::emptyValue().asReturnedValue();
     return s->values[index].asReturnedValue();
 }
 
@@ -468,14 +412,14 @@ bool SparseArrayData::del(Object *o, uint index)
 
     if (isAccessor) {
         // free up both indices
-        dd->values.values[pidx + 1].setEmpty(Value::fromReturnedValue(dd->freeList).emptyValue());
-        dd->values.values[pidx].setEmpty(pidx + 1);
+        dd->values.values[pidx + 1] = dd->sparse->freeList;
+        dd->values.values[pidx] = Encode(pidx + 1);
     } else {
         Q_ASSERT(dd->type == Heap::ArrayData::Sparse);
-        dd->values.values[pidx].setEmpty(Value::fromReturnedValue(dd->freeList).emptyValue());
+        dd->values.values[pidx] = dd->sparse->freeList;
     }
 
-    dd->freeList = Primitive::emptyValue(pidx).asReturnedValue();
+    dd->sparse->freeList = Encode(pidx);
     dd->sparse->erase(n);
     return true;
 }
@@ -587,8 +531,8 @@ uint ArrayData::append(Object *obj, ArrayObject *otherObj, uint n)
     if (!other || ArgumentsObject::isNonStrictArgumentsObject(otherObj)) {
         ScopedValue v(scope);
         for (uint i = 0; i < n; ++i)
-            obj->arraySet(oldSize + i, (v = otherObj->getIndexed(i)));
-    } else if (other && other->isSparse()) {
+            obj->arraySet(oldSize + i, (v = otherObj->get(i)));
+    } else if (other->isSparse()) {
         Heap::SparseArrayData *os = static_cast<Heap::SparseArrayData *>(other->d());
         if (other->hasAttributes()) {
             ScopedValue v(scope);
@@ -607,7 +551,7 @@ uint ArrayData::append(Object *obj, ArrayObject *otherObj, uint n)
         uint toCopy = n;
         uint chunk = toCopy;
         if (chunk > os->values.alloc - os->offset)
-            chunk -= os->values.alloc - os->offset;
+            chunk = os->values.alloc - os->offset;
         obj->arrayPut(oldSize, os->values.data() + os->offset, chunk);
         toCopy -= chunk;
         if (toCopy)
@@ -629,7 +573,7 @@ void ArrayData::insert(Object *o, uint index, const Value *v, bool isAccessor)
             if (index >= d->values.size) {
                 // mark possible hole in the array
                 for (uint i = d->values.size; i < index; ++i)
-                    d->setData(o->engine(), i, Primitive::emptyValue());
+                    d->setData(o->engine(), i, Value::emptyValue());
                 d->values.size = index + 1;
             }
             d->setData(o->engine(), index, *v);
@@ -648,22 +592,6 @@ void ArrayData::insert(Object *o, uint index, const Value *v, bool isAccessor)
         s->setArrayData(o->engine(), n->value + Object::SetterOffset, v[Object::SetterOffset]);
 }
 
-
-class ArrayElementLessThan
-{
-public:
-    inline ArrayElementLessThan(ExecutionEngine *engine, Object *thisObject, const Value &comparefn)
-        : m_engine(engine), thisObject(thisObject), m_comparefn(comparefn) {}
-
-    bool operator()(Value v1, Value v2) const;
-
-private:
-    ExecutionEngine *m_engine;
-    Object *thisObject;
-    const Value &m_comparefn;
-};
-
-
 bool ArrayElementLessThan::operator()(Value v1, Value v2) const
 {
     Scope scope(m_engine);
@@ -672,77 +600,30 @@ bool ArrayElementLessThan::operator()(Value v1, Value v2) const
         return false;
     if (v2.isUndefined() || v2.isEmpty())
         return true;
-    ScopedObject o(scope, m_comparefn);
+    ScopedFunctionObject o(scope, m_comparefn);
     if (o) {
         Scope scope(o->engine());
         ScopedValue result(scope);
-        ScopedCallData callData(scope, 2);
-        callData->thisObject = Primitive::undefinedValue();
-        callData->args[0] = v1;
-        callData->args[1] = v2;
-        result = QV4::Runtime::method_callValue(scope.engine, m_comparefn, callData);
+        JSCallArguments jsCallData(scope, 2);
+        jsCallData.args[0] = v1;
+        jsCallData.args[1] = v2;
+        result = o->call(jsCallData);
+        if (scope.hasException())
+            return false;
 
         return result->toNumber() < 0;
     }
     ScopedString p1s(scope, v1.toString(scope.engine));
     ScopedString p2s(scope, v2.toString(scope.engine));
+
+    if (!p1s)
+        return false;
+    if (!p2s)
+        return true;
+
     return p1s->toQString() < p2s->toQString();
 }
 
-template <typename RandomAccessIterator, typename T, typename LessThan>
-void sortHelper(RandomAccessIterator start, RandomAccessIterator end, const T &t, LessThan lessThan)
-{
-top:
-    int span = int(end - start);
-    if (span < 2)
-        return;
-
-    --end;
-    RandomAccessIterator low = start, high = end - 1;
-    RandomAccessIterator pivot = start + span / 2;
-
-    if (lessThan(*end, *start))
-        qSwap(*end, *start);
-    if (span == 2)
-        return;
-
-    if (lessThan(*pivot, *start))
-        qSwap(*pivot, *start);
-    if (lessThan(*end, *pivot))
-        qSwap(*end, *pivot);
-    if (span == 3)
-        return;
-
-    qSwap(*pivot, *end);
-
-    while (low < high) {
-        while (low < high && lessThan(*low, *end))
-            ++low;
-
-        while (high > low && lessThan(*end, *high))
-            --high;
-
-        if (low < high) {
-            qSwap(*low, *high);
-            ++low;
-            --high;
-        } else {
-            break;
-        }
-    }
-
-    if (lessThan(*low, *end))
-        ++low;
-
-    qSwap(*end, *low);
-    sortHelper(start, low, t, lessThan);
-
-    start = low + 1;
-    ++end;
-    goto top;
-}
-
-
 void ArrayData::sort(ExecutionEngine *engine, Object *thisObject, const Value &comparefn, uint len)
 {
     if (!len)
@@ -754,11 +635,6 @@ void ArrayData::sort(ExecutionEngine *engine, Object *thisObject, const Value &c
     if (!arrayData || !arrayData->length())
         return;
 
-    if (!(comparefn.isUndefined() || comparefn.as<Object>())) {
-        engine->throwTypeError();
-        return;
-    }
-
     // The spec says the sorting goes through a series of get,put and delete operations.
     // this implies that the attributes don't get sorted around.
 
@@ -770,7 +646,7 @@ void ArrayData::sort(ExecutionEngine *engine, Object *thisObject, const Value &c
         if (!sparse->sparse()->nEntries())
             return;
 
-        thisObject->setArrayData(0);
+        thisObject->setArrayData(nullptr);
         ArrayData::realloc(thisObject, Heap::ArrayData::Simple, sparse->sparse()->nEntries(), sparse->attrs() ? true : false);
         Heap::SimpleArrayData *d = thisObject->d()->arrayData.cast<Heap::SimpleArrayData>();
 
@@ -824,7 +700,7 @@ void ArrayData::sort(ExecutionEngine *engine, Object *thisObject, const Value &c
                         break;
                 Q_ASSERT(!d->attrs || !d->attrs[len].isAccessor());
                 d->setData(engine, i, d->data(len));
-                d->setData(engine, len, Primitive::emptyValue());
+                d->setData(engine, len, Value::emptyValue());
             }
         }
 
@@ -833,10 +709,38 @@ void ArrayData::sort(ExecutionEngine *engine, Object *thisObject, const Value &c
     }
 
 
-    ArrayElementLessThan lessThan(engine, thisObject, comparefn);
+    ArrayElementLessThan lessThan(engine, comparefn);
 
-    Value *begin = thisObject->arrayData()->values.values;
-    sortHelper(begin, begin + len, *begin, lessThan);
+    const auto thisArrayData = thisObject->arrayData();
+    uint startIndex = thisArrayData->mappedIndex(0);
+    uint endIndex = thisArrayData->mappedIndex(len - 1) + 1;
+    if (startIndex < endIndex) {
+        // Values are contiguous. Sort right away.
+        sortHelper(
+                thisArrayData->values.values + startIndex,
+                thisArrayData->values.values + endIndex,
+                lessThan);
+    } else {
+        // Values wrap around the end of the allocation. Close the gap to form a contiguous array.
+        // We're going to sort anyway. So we don't need to care about order.
+
+        // ArrayElementLessThan sorts empty and undefined to the end of the array anyway, but we
+        // probably shouldn't rely on the unused slots to be actually undefined or empty.
+
+        const uint gap = startIndex - endIndex;
+        const uint allocEnd = thisArrayData->values.alloc - 1;
+        for (uint i = 0; i < gap; ++i) {
+            const uint from = allocEnd - i;
+            const uint to = endIndex + i;
+            if (from < startIndex)
+                break;
+
+            std::swap(thisArrayData->values.values[from], thisArrayData->values.values[to]);
+        }
+
+        thisArrayData->offset = 0;
+        sortHelper(thisArrayData->values.values, thisArrayData->values.values + len, lessThan);
+    }
 
 #ifdef CHECK_SPARSE_ARRAYS
     thisObject->initSparseArray();
