@@ -207,6 +207,7 @@ public:
         URIErrorProto,
         PromiseProto,
         VariantProto,
+        VariantAssociationProto,
         SequenceProto,
         SharedArrayBufferProto,
         ArrayBufferProto,
@@ -329,6 +330,7 @@ public:
     Object *uRIErrorPrototype() const { return reinterpret_cast<Object *>(jsObjects + URIErrorProto); }
     Object *promisePrototype() const { return reinterpret_cast<Object *>(jsObjects + PromiseProto); }
     Object *variantPrototype() const { return reinterpret_cast<Object *>(jsObjects + VariantProto); }
+    Object *variantAssociationPrototype() const { return reinterpret_cast<Object *>(jsObjects + VariantAssociationProto); }
     Object *sequencePrototype() const { return reinterpret_cast<Object *>(jsObjects + SequenceProto); }
 
     Object *sharedArrayBufferPrototype() const { return reinterpret_cast<Object *>(jsObjects + SharedArrayBufferProto); }
@@ -673,6 +675,8 @@ public:
     // Use only inside catch(...) -- will re-throw if no JS exception
     QQmlError catchExceptionAsQmlError();
 
+    void amendException();
+
     // variant conversions
     static QVariant toVariant(
         const QV4::Value &value, QMetaType typeHint, bool createJSValueForObjectsAndSymbols = true);
@@ -682,6 +686,7 @@ public:
             const QVariant &variant, Heap::Object *parent, int property, uint flags);
 
     static QVariantMap variantMapFromJS(const QV4::Object *o);
+    static QVariantHash variantHashFromJS(const QV4::Object *o);
 
     static bool metaTypeFromJS(const Value &value, QMetaType type, void *data);
     QV4::ReturnedValue metaTypeToJS(QMetaType type, const void *data);
@@ -769,17 +774,13 @@ public:
     {
         return m_compilationUnits;
     }
+
     void trimCompilationUnits();
+    void trimCompilationUnitsForUrl(const QUrl &url);
 
-    QV4::Value *registerNativeModule(const QUrl &url, const QV4::Value &module);
-
-    struct Module {
-        QQmlRefPointer<ExecutableCompilationUnit> compiled;
-
-        // We can pass a raw value pointer here, but nowhere else. See below.
-        Value *native = nullptr;
-    };
+    using Module = QQmlRefPointer<ExecutableCompilationUnit>;
 
+    Module registerNativeModule(const QUrl &url, const QV4::Value &value);
     Module moduleForUrl(const QUrl &_url, const ExecutableCompilationUnit *referrer = nullptr) const;
     Module loadModule(const QUrl &_url, const ExecutableCompilationUnit *referrer = nullptr);
 
@@ -816,14 +817,6 @@ public:
         return ok ? QJSPrimitiveValue(result) : QJSPrimitiveValue(QJSPrimitiveUndefined());
     }
 
-    ReturnedValue nativeModule(const QUrl &url) const
-    {
-        const auto it = nativeModules.find(url);
-        return it == nativeModules.end()
-                ? QV4::Value::emptyValue().asReturnedValue()
-                : (*it)->asReturnedValue();
-    }
-
 private:
     template<int Frames>
     friend struct ExecutionEngineCallDepthRecorder;
@@ -895,12 +888,6 @@ private:
     QVector<Deletable *> m_extensionData;
 
     QMultiHash<QUrl, QQmlRefPointer<ExecutableCompilationUnit>> m_compilationUnits;
-
-    // QV4::PersistentValue would be preferred, but using QHash will create copies,
-    // and QV4::PersistentValue doesn't like creating copies.
-    // Instead, we allocate a raw pointer using the same manual memory management
-    // technique in QV4::PersistentValue.
-    QHash<QUrl, Value *> nativeModules;
 };
 
 #define CHECK_STACK_LIMITS(v4) \
