@@ -34,77 +34,82 @@
 #include <QScopedValueRollback>
 #include <QLoggingCategory>
 
-Q_LOGGING_CATEGORY(lcQmlDefaultMethod, "qt.qml.defaultmethod")
+Q_STATIC_LOGGING_CATEGORY(lcQmlDefaultMethod, "qt.qml.defaultmethod")
 
 QT_USE_NAMESPACE
 
 Q_TRACE_PREFIX(qtqml,
 "namespace QV4 {" \
 "struct ExecutionEngine;" \
+"class ExecutableCompilationUnit;" \
 "namespace CompiledData {" \
-"struct CompilationUnit;" \
 "struct Object;" \
 "}}" \
 "class QQmlEngine;"
 )
 
-Q_TRACE_POINT(qtqml, QQmlObjectCreator_createInstance_entry, const QV4::CompiledData::CompilationUnit *compilationUnit, const QV4::CompiledData::Object *object, const QUrl &url)
+Q_TRACE_POINT(qtqml, QQmlObjectCreator_createInstance_entry, const QV4::ExecutableCompilationUnit *compilationUnit, const QV4::CompiledData::Object *object, const QUrl &url)
 Q_TRACE_POINT(qtqml, QQmlObjectCreator_createInstance_exit, const QString &typeName)
 
 QQmlObjectCreator::QQmlObjectCreator(
-        QQmlRefPointer<QQmlContextData> parentContext,
+        const QQmlRefPointer<QQmlContextData> &parentContext,
         const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit,
         const QQmlRefPointer<QQmlContextData> &creationContext,
+        const QString &inlineComponentName,
         QQmlIncubatorPrivate *incubator)
     : phase(Startup)
+    , m_inlineComponentName(inlineComponentName)
     , compilationUnit(compilationUnit)
-    , propertyCaches(&compilationUnit->propertyCaches)
+    , propertyCaches(compilationUnit->propertyCachesPtr())
     , sharedState(new QQmlObjectCreatorSharedState, QQmlRefPointer<QQmlObjectCreatorSharedState>::Adopt)
     , topLevelCreator(true)
     , isContextObject(true)
     , incubator(incubator)
 {
-    init(std::move(parentContext));
+    init(parentContext);
 
     sharedState->componentAttached = nullptr;
-    sharedState->allCreatedBindings.allocate(compilationUnit->totalBindingsCount());
-    sharedState->allParserStatusCallbacks.allocate(compilationUnit->totalParserStatusCount());
-    sharedState->allCreatedObjects.allocate(compilationUnit->totalObjectCount());
-    sharedState->allJavaScriptObjects = nullptr;
+    sharedState->allCreatedBindings.allocate(compilationUnit->totalBindingsCount(inlineComponentName));
+    sharedState->allParserStatusCallbacks.allocate(compilationUnit->totalParserStatusCount(inlineComponentName));
+    sharedState->allCreatedObjects.allocate(compilationUnit->totalObjectCount(inlineComponentName));
+    sharedState->allJavaScriptObjects = ObjectInCreationGCAnchorList();
     sharedState->creationContext = creationContext;
     sharedState->rootContext.reset();
     sharedState->hadTopLevelRequiredProperties = false;
 
     if (auto profiler = QQmlEnginePrivate::get(engine)->profiler) {
         Q_QML_PROFILE_IF_ENABLED(QQmlProfilerDefinitions::ProfileCreating, profiler,
-                sharedState->profiler.init(profiler, compilationUnit->totalParserStatusCount()));
+                sharedState->profiler.init(profiler, compilationUnit->totalParserStatusCount(inlineComponentName)));
     } else {
         Q_UNUSED(profiler);
     }
 }
 
-QQmlObjectCreator::QQmlObjectCreator(QQmlRefPointer<QQmlContextData> parentContext,
-        const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit,
-        QQmlObjectCreatorSharedState *inheritedSharedState, bool isContextObject)
+QQmlObjectCreator::QQmlObjectCreator(const QQmlRefPointer<QQmlContextData> &parentContext,
+                                     const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit, const QString inlineComponentName,
+                                     QQmlObjectCreatorSharedState *inheritedSharedState, bool isContextObject)
     : phase(Startup)
+    , m_inlineComponentName(inlineComponentName)
     , compilationUnit(compilationUnit)
-    , propertyCaches(&compilationUnit->propertyCaches)
+    , propertyCaches(compilationUnit->propertyCachesPtr())
     , sharedState(inheritedSharedState)
     , topLevelCreator(false)
     , isContextObject(isContextObject)
     , incubator(nullptr)
 {
-    init(std::move(parentContext));
+    init(parentContext);
 }
 
-void QQmlObjectCreator::init(QQmlRefPointer<QQmlContextData> providedParentContext)
+void QQmlObjectCreator::init(const QQmlRefPointer<QQmlContextData> &providedParentContext)
 {
-    parentContext = std::move(providedParentContext);
+    parentContext = providedParentContext;
     engine = parentContext->engine();
     v4 = engine->handle();
 
-    if (compilationUnit && !compilationUnit->engine)
-        compilationUnit->linkToEngine(v4);
+    Q_ASSERT(compilationUnit);
+    Q_ASSERT(compilationUnit->engine == v4);
+    if (!compilationUnit->runtimeStrings)
+        compilationUnit->populate();
 
     qmlUnit = compilationUnit->unitData();
     _qobject = nullptr;
@@ -187,13 +192,14 @@ QObject *QQmlObjectCreator::create(int subComponentIndex, QObject *parent, QQmlI
 
     QV4::Scope scope(v4);
 
-    Q_ASSERT(sharedState->allJavaScriptObjects || topLevelCreator);
+    Q_ASSERT(sharedState->allJavaScriptObjects.canTrack() || topLevelCreator);
     if (topLevelCreator)
-        sharedState->allJavaScriptObjects = scope.alloc(compilationUnit->totalObjectCount());
+        sharedState->allJavaScriptObjects = ObjectInCreationGCAnchorList(scope, compilationUnit->totalObjectCount(m_inlineComponentName));
 
     if (!isComponentRoot && sharedState->creationContext) {
         // otherwise QQmlEnginePrivate::createInternalContext() handles it
-        context->setImportedScripts(sharedState->creationContext->importedScripts());
+        QV4::ScopedValue scripts(scope, sharedState->creationContext->importedScripts());
+        context->setImportedScripts(v4, scripts);
     }
 
     QObject *instance = createInstance(objectToCreate, parent, /*isContextObject*/true);
@@ -204,7 +210,7 @@ QObject *QQmlObjectCreator::create(int subComponentIndex, QObject *parent, QQmlI
     }
 
     if (topLevelCreator)
-        sharedState->allJavaScriptObjects = nullptr;
+        sharedState->allJavaScriptObjects = ObjectInCreationGCAnchorList();
 
     phase = CreatingObjectsPhase2;
 
@@ -233,10 +239,11 @@ void QQmlObjectCreator::beginPopulateDeferred(const QQmlRefPointer<QQmlContextDa
     sharedState->rootContext = newContext;
 
     Q_ASSERT(topLevelCreator);
-    Q_ASSERT(!sharedState->allJavaScriptObjects);
+    Q_ASSERT(!sharedState->allJavaScriptObjects.canTrack());
 
+    // FIXME (QTBUG-122956): allocating from the short lived scope does not make any sense
     QV4::Scope valueScope(v4);
-    sharedState->allJavaScriptObjects = valueScope.alloc(compilationUnit->totalObjectCount());
+    sharedState->allJavaScriptObjects = ObjectInCreationGCAnchorList(valueScope, compilationUnit->totalObjectCount(m_inlineComponentName));
 }
 
 void QQmlObjectCreator::populateDeferred(QObject *instance, int deferredIndex,
@@ -668,7 +675,8 @@ void QQmlObjectCreator::setPropertyValue(const QQmlPropertyData *property, const
                 break;
             }
 
-            QVariant target = QQmlValueTypeProvider::createValueType(source, propertyType);
+            QVariant target = QQmlValueTypeProvider::createValueType(
+                    source, propertyType, engine->handle());
             if (target.isValid()) {
                 property->writeProperty(_qobject, target.data(), propertyWriteFlags);
                 break;
@@ -701,10 +709,11 @@ void QQmlObjectCreator::setupBindings(BindingSetupFlags mode)
     QQmlListProperty<void> savedList;
     qSwap(_currentList, savedList);
 
-    const QV4::BindingPropertyData &propertyData = compilationUnit->bindingPropertyDataPerObject.at(_compiledObjectIndex);
+    const QV4::CompiledData::BindingPropertyData *propertyData
+            = compilationUnit->bindingPropertyDataPerObjectAt(_compiledObjectIndex);
 
     if (_compiledObject->idNameIndex) {
-        const QQmlPropertyData *idProperty = propertyData.last();
+        const QQmlPropertyData *idProperty = propertyData->last();
         Q_ASSERT(!idProperty || !idProperty->isValid() || idProperty->name(_qobject) == QLatin1String("id"));
         if (idProperty && idProperty->isValid() && idProperty->isWritable() && idProperty->propType().id() == QMetaType::QString) {
             QV4::CompiledData::Binding idBinding;
@@ -751,7 +760,7 @@ void QQmlObjectCreator::setupBindings(BindingSetupFlags mode)
 
     const QV4::CompiledData::Binding *binding = _compiledObject->bindingTable();
     for (quint32 i = 0; i < _compiledObject->nBindings; ++i, ++binding) {
-        const QQmlPropertyData *const property = propertyData.at(i);
+        const QQmlPropertyData *const property = propertyData->at(i);
         if (property) {
             const QQmlPropertyData *targetProperty = property;
             if (targetProperty->isAlias()) {
@@ -832,16 +841,17 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
         QV4::ResolvedTypeReference *tr = resolvedType(binding->propertyNameIndex);
         Q_ASSERT(tr);
         QQmlType attachedType = tr->type();
+        QQmlEnginePrivate *enginePrivate = QQmlEnginePrivate::get(engine);
         if (!attachedType.isValid()) {
             QQmlTypeNameCache::Result res = context->imports()->query(
-                        stringAt(binding->propertyNameIndex));
+                    stringAt(binding->propertyNameIndex), QQmlTypeLoader::get(enginePrivate));
             if (res.isValid())
                 attachedType = res.type;
             else
                 return false;
         }
         QObject *qmlObject = qmlAttachedPropertiesObject(
-                _qobject, attachedType.attachedPropertiesFunction(QQmlEnginePrivate::get(engine)));
+                _qobject, attachedType.attachedPropertiesFunction(enginePrivate));
         if (!qmlObject) {
             recordError(binding->location,
                         QStringLiteral("Could not create attached properties object '%1'")
@@ -849,6 +859,21 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
             return false;
         }
 
+        const QQmlType attachedObjectType = QQmlMetaType::qmlType(attachedType.attachedPropertiesType(QQmlEnginePrivate::get((engine))));
+        const int parserStatusCast = attachedObjectType.parserStatusCast();
+        QQmlParserStatus *parserStatus  = nullptr;
+        if (parserStatusCast != -1)
+            parserStatus = reinterpret_cast<QQmlParserStatus*>(reinterpret_cast<char *>(qmlObject) + parserStatusCast);
+        if (parserStatus) {
+            parserStatus->classBegin();
+            // we ignore them for profiling, because it doesn't interact well with the logic anyway
+            // can't use sharedState->allParserStatusCallbacks, we haven't reserved space TODO: could we?
+            if (!sharedState->attachedObjectParserStatusCallbacks)
+                sharedState->attachedObjectParserStatusCallbacks = std::make_unique<std::deque<QQmlParserStatus *>>();
+            sharedState->attachedObjectParserStatusCallbacks->push_back(parserStatus);
+            parserStatus->d = &sharedState->attachedObjectParserStatusCallbacks->back();
+        }
+
         if (!populateInstance(binding->value.objectIndex, qmlObject, qmlObject,
                               /*value type property*/ nullptr, binding))
             return false;
@@ -876,6 +901,9 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
 
     QObject *createdSubObject = nullptr;
     if (bindingType == QV4::CompiledData::Binding::Type_Object) {
+        // This is not a top level object. Its required properties don't count towards the
+        // top level required properties.
+        QScopedValueRollback topLevelRequired(sharedState->hadTopLevelRequiredProperties);
         createdSubObject = createInstance(binding->value.objectIndex, _bindingTarget);
         if (!createdSubObject)
             return false;
@@ -956,7 +984,7 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
             && !_valueTypeProperty;
 
     if (allowedToRemoveBinding) {
-        if (bindingProperty->isBindable()) {
+        if (bindingProperty->acceptsQBinding()) {
             removePendingBinding(_bindingTarget, bindingProperty->coreIndex());
         } else {
             QQmlPropertyPrivate::removeBinding(
@@ -973,7 +1001,7 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
                         _bindingTarget, signalIndex, context,
                         _scopeObject, runtimeFunction, currentQmlContext());
 
-            if (bindingProperty->isBindable()) {
+            if (bindingProperty->notifiesViaBindable()) {
                 auto target = _bindingTarget;
                 if (bindingProperty->isAlias()) {
                     // If the property is an alias, we cannot obtain the bindable interface directly with qt_metacall
@@ -997,7 +1025,7 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
                 QQmlBoundSignal *bs = new QQmlBoundSignal(_bindingTarget, signalIndex, _scopeObject, engine);
                 bs->takeExpression(expr);
             }
-        } else if (bindingProperty->isBindable()) {
+        } else if (bindingProperty->acceptsQBinding()) {
             QUntypedPropertyBinding qmlBinding;
             if (binding->isTranslationBinding()) {
                 qmlBinding = QQmlTranslationPropertyBinding::create(bindingProperty, compilationUnit, binding);
@@ -1135,9 +1163,9 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
                 recordError(binding->valueLocation, tr("Cannot assign object type %1 with no default method").arg(QString::fromLatin1(createdSubObject->metaObject()->className())));
                 return false;
             }
-            qCWarning(lcQmlDefaultMethod) << "Assigning an object to a signal handler is deprecated."
-                                             "Instead, create the object, give it an id, and call the desired slot from the signal handler."
-                                             ;
+            qCWarning(lcQmlDefaultMethod) << "Assigning an object to a signal handler is deprecated. "
+                                             "Instead, create the object, give it an id, and call the desired slot "
+                                             "from the signal handler. The object is:" << createdSubObject;
 
             QMetaMethod signalMethod = _qobject->metaObject()->method(bindingProperty->coreIndex());
             if (!QMetaObject::checkConnectArgs(signalMethod, method)) {
@@ -1205,11 +1233,17 @@ bool QQmlObjectCreator::setPropertyBinding(const QQmlPropertyData *bindingProper
                 recordError(binding->location, tr("Cannot assign object to read only list"));
                 return false;
             }
-
-        } else {
+        } else if (bindingProperty->propType().flags().testFlag(QMetaType::PointerToQObject)) {
             // pointer compatibility was tested in QQmlPropertyValidator at type compile time
             argv[0] = &createdSubObject;
             QMetaObject::metacall(_qobject, QMetaObject::WriteProperty, bindingProperty->coreIndex(), argv);
+        } else {
+            QVariant target = QQmlValueTypeProvider::createValueType(
+                    QVariant::fromValue(createdSubObject), bindingProperty->propType(), v4);
+            if (target.isValid())
+                bindingProperty->writeProperty(_qobject, target.data(), propertyWriteFlags);
+            else
+                recordError(binding->location, tr("Cannot construct value type from given object"));
         }
         return true;
     }
@@ -1318,7 +1352,7 @@ QObject *QQmlObjectCreator::createInstance(int index, QObject *parent, bool isCo
 
             sharedState->allCreatedObjects.push(instance);
         } else {
-            const auto compilationUnit = typeRef->compilationUnit();
+            auto compilationUnit = typeRef->compilationUnit();
             Q_ASSERT(compilationUnit);
             typeName = compilationUnit->fileName();
             // compilation unit is shared between root type and its inline component types
@@ -1329,34 +1363,31 @@ QObject *QQmlObjectCreator::createInstance(int index, QObject *parent, bool isCo
             }
 
             if (!type.isInlineComponentType()) {
-                QQmlObjectCreator subCreator(context, compilationUnit, sharedState.data(),
-                                             isContextObject);
+                QQmlObjectCreator subCreator(
+                        context, engine->handle()->executableCompilationUnit(
+                                         std::move(compilationUnit)),
+                        QString(), // not an inline component
+                        sharedState.data(), isContextObject);
                 instance = subCreator.create();
                 if (!instance) {
                     errors += subCreator.errors;
                     return nullptr;
                 }
             } else {
-                QString subObjectName;
-                if (compilationUnit->icRootName) {
-                    subObjectName = type.elementName();
-                    std::swap(*compilationUnit->icRootName, subObjectName);
-                } else {
-                    compilationUnit->icRootName = std::make_unique<QString>(type.elementName());
-                }
+                const QString inlineComponentName = type.elementName();
 
-                const auto guard = qScopeGuard([&] {
-                    if (subObjectName.isEmpty())
-                        compilationUnit->icRootName.reset();
-                    else
-                        std::swap(*compilationUnit->icRootName, subObjectName);
-                });
-
-                QQmlObjectCreator subCreator(context, compilationUnit, sharedState.data(),
-                                             isContextObject);
+                const int inlineComponentId
+                        = compilationUnit->inlineComponentId(inlineComponentName);
+                QQmlObjectCreator subCreator(
+                        context,
+                        engine->handle()->executableCompilationUnit(
+                                QQmlRefPointer<QV4::CompiledData::CompilationUnit>(
+                                        compilationUnit)),
+                        inlineComponentName,
+                        sharedState.data(),
+                        isContextObject);
                 instance = subCreator.create(
-                    compilationUnit->inlineComponentId(*compilationUnit->icRootName),
-                    nullptr, nullptr, CreationFlags::InlineComponent);
+                        inlineComponentId, nullptr, nullptr, CreationFlags::InlineComponent);
                 if (!instance) {
                     errors += subCreator.errors;
                     return nullptr;
@@ -1409,7 +1440,7 @@ QObject *QQmlObjectCreator::createInstance(int index, QObject *parent, bool isCo
 
     if (customParser && obj->hasFlag(QV4::CompiledData::Object::HasCustomParserBindings)) {
         customParser->engine = QQmlEnginePrivate::get(engine);
-        customParser->imports = compilationUnit->typeNameCache.data();
+        customParser->imports = compilationUnit->typeNameCache().data();
 
         QList<const QV4::CompiledData::Binding *> bindings;
         const QV4::CompiledData::Object *obj = compilationUnit->objectAt(index);
@@ -1437,9 +1468,8 @@ QObject *QQmlObjectCreator::createInstance(int index, QObject *parent, bool isCo
     QObject *scopeObject = instance;
     qSwap(_scopeObject, scopeObject);
 
-    Q_ASSERT(sharedState->allJavaScriptObjects);
-    *sharedState->allJavaScriptObjects = QV4::QObjectWrapper::wrap(v4, instance);
-    ++sharedState->allJavaScriptObjects;
+    Q_ASSERT(sharedState->allJavaScriptObjects.canTrack());
+    sharedState->allJavaScriptObjects.trackObject(v4, instance);
 
     QV4::Scope valueScope(v4);
     QV4::QmlContext *qmlContext = static_cast<QV4::QmlContext *>(valueScope.alloc());
@@ -1535,7 +1565,7 @@ bool QQmlObjectCreator::finalize(QQmlInstantiationInterrupt &interrupt)
         target->metaObject()->metacall(target, QMetaObject::BindableProperty, index, argv);
         const bool success = bindable.setBinding(qmlBinding);
 
-        const auto bindingPrivateRefCount = QPropertyBindingPrivate::get(qmlBinding)->ref;
+        const auto bindingPrivateRefCount = QPropertyBindingPrivate::get(qmlBinding)->refCount();
 
         // Only pop_front after setting the binding as the bindings are refcounted.
         sharedState->allQPropertyBindings.pop_front();
@@ -1567,6 +1597,19 @@ bool QQmlObjectCreator::finalize(QQmlInstantiationInterrupt &interrupt)
                 status->componentComplete();
             }
 
+            if (watcher.hasRecursed() || interrupt.shouldInterrupt())
+                return false;
+        }
+        while (sharedState->attachedObjectParserStatusCallbacks && !sharedState->attachedObjectParserStatusCallbacks->empty()) {
+            // ### TODO: no profiler integration (QTBUG-132827)
+            QQmlParserStatus *status = sharedState->attachedObjectParserStatusCallbacks->back();
+            sharedState->attachedObjectParserStatusCallbacks->pop_back();
+
+            if (status && status->d) {
+                status->d = nullptr;
+                status->componentComplete();
+            }
+
             if (watcher.hasRecursed() || interrupt.shouldInterrupt())
                 return false;
         }
@@ -1657,7 +1700,7 @@ bool QQmlObjectCreator::populateInstance(int index, QObject *instance, QObject *
 
     _ddata->compilationUnit = compilationUnit;
     if (_compiledObject->hasFlag(QV4::CompiledData::Object::HasDeferredBindings))
-        _ddata->deferData(_compiledObjectIndex, compilationUnit, context);
+        _ddata->deferData(_compiledObjectIndex, compilationUnit, context, m_inlineComponentName);
 
     const qsizetype oldRequiredPropertiesCount = sharedState->requiredProperties.size();
     QSet<QString> postHocRequired;
@@ -1840,3 +1883,15 @@ QQmlObjectCreatorRecursionWatcher::QQmlObjectCreatorRecursionWatcher(QQmlObjectC
     , watcher(creator->sharedState.data())
 {
 }
+
+void ObjectInCreationGCAnchorList::trackObject(QV4::ExecutionEngine *engine, QObject *instance)
+{
+    *allJavaScriptObjects = QV4::QObjectWrapper::wrap(engine, instance);
+    // we have to handle the case where the gc is already running, but the scope is discarded
+    // before the collector runs again. In that case, rescanning won't help us. Thus, mark the
+    // object.
+    QV4::WriteBarrier::markCustom(engine, [this](QV4::MarkStack *ms) {
+        allJavaScriptObjects->heapObject()->mark(ms);
+    });
+    ++allJavaScriptObjects;
+}
