@@ -7,10 +7,14 @@
 #include <qsgmaterialshader.h>
 #include <qsgtextureprovider.h>
 #include <private/qsgplaintexture_p.h>
-#include <QtGui/private/qshaderdescription_p.h>
+#include <rhi/qshaderdescription.h>
 #include <QQmlFile>
+
+#include <QtGui/qquaternion.h>
+
 #include <QFile>
 #include <QFileSelector>
+#include <QMutexLocker>
 
 QT_BEGIN_NAMESPACE
 
@@ -22,19 +26,23 @@ void QSGRhiShaderLinker::reset(const QShader &vs, const QShader &fs)
 
     m_error = false;
 
-    m_constantBufferSize = 0;
+    //m_constantBufferSize = 0;
     m_constants.clear();
     m_samplers.clear();
     m_samplerNameMap.clear();
     m_subRectBindings.clear();
+
+    m_constants.reserve(8);
+    m_samplers.reserve(4);
+    m_samplerNameMap.reserve(4);
 }
 
 void QSGRhiShaderLinker::feedConstants(const QSGShaderEffectNode::ShaderData &shader, const QSet<int> *dirtyIndices)
 {
-    Q_ASSERT(shader.shaderInfo.variables.count() == shader.varData.count());
+    Q_ASSERT(shader.shaderInfo.variables.size() == shader.varData.size());
+    static bool shaderEffectDebug = qEnvironmentVariableIntValue("QSG_RHI_SHADEREFFECT_DEBUG");
     if (!dirtyIndices) {
-        m_constantBufferSize = qMax(m_constantBufferSize, shader.shaderInfo.constantDataSize);
-        for (int i = 0; i < shader.shaderInfo.variables.count(); ++i) {
+        for (int i = 0; i < shader.shaderInfo.variables.size(); ++i) {
             const QSGGuiThreadShaderEffectManager::ShaderInfo::Variable &var(shader.shaderInfo.variables.at(i));
             if (var.type == QSGGuiThreadShaderEffectManager::ShaderInfo::Constant) {
                 const QSGShaderEffectNode::VariableData &vd(shader.varData.at(i));
@@ -43,7 +51,7 @@ void QSGRhiShaderLinker::feedConstants(const QSGShaderEffectNode::ShaderData &sh
                 c.specialType = vd.specialType;
                 if (c.specialType != QSGShaderEffectNode::VariableData::SubRect) {
                     c.value = vd.value;
-                    if (QSGRhiSupport::instance()->isShaderEffectDebuggingRequested()) {
+                    if (shaderEffectDebug) {
                         if (c.specialType == QSGShaderEffectNode::VariableData::None) {
                             qDebug() << "cbuf prepare" << shader.shaderInfo.name << var.name
                                      << "offset" << var.offset << "value" << c.value;
@@ -64,7 +72,7 @@ void QSGRhiShaderLinker::feedConstants(const QSGShaderEffectNode::ShaderData &sh
             const int offset = shader.shaderInfo.variables.at(idx).offset;
             const QVariant value = shader.varData.at(idx).value;
             m_constants[offset].value = value;
-            if (QSGRhiSupport::instance()->isShaderEffectDebuggingRequested()) {
+            if (shaderEffectDebug) {
                 qDebug() << "cbuf update" << shader.shaderInfo.name
                          << "offset" << offset << "value" << value;
             }
@@ -75,11 +83,17 @@ void QSGRhiShaderLinker::feedConstants(const QSGShaderEffectNode::ShaderData &sh
 void QSGRhiShaderLinker::feedSamplers(const QSGShaderEffectNode::ShaderData &shader, const QSet<int> *dirtyIndices)
 {
     if (!dirtyIndices) {
-        for (int i = 0; i < shader.shaderInfo.variables.count(); ++i) {
+        for (int i = 0; i < shader.shaderInfo.variables.size(); ++i) {
             const QSGGuiThreadShaderEffectManager::ShaderInfo::Variable &var(shader.shaderInfo.variables.at(i));
             const QSGShaderEffectNode::VariableData &vd(shader.varData.at(i));
             if (var.type == QSGGuiThreadShaderEffectManager::ShaderInfo::Sampler) {
                 Q_ASSERT(vd.specialType == QSGShaderEffectNode::VariableData::Source);
+
+#ifndef QT_NO_DEBUG
+                int existingBindPoint = m_samplerNameMap.value(var.name, -1);
+                Q_ASSERT(existingBindPoint < 0 || existingBindPoint == var.bindPoint);
+#endif
+
                 m_samplers.insert(var.bindPoint, vd.value);
                 m_samplerNameMap.insert(var.name, var.bindPoint);
             }
@@ -88,6 +102,12 @@ void QSGRhiShaderLinker::feedSamplers(const QSGShaderEffectNode::ShaderData &sha
         for (int idx : *dirtyIndices) {
             const QSGGuiThreadShaderEffectManager::ShaderInfo::Variable &var(shader.shaderInfo.variables.at(idx));
             const QSGShaderEffectNode::VariableData &vd(shader.varData.at(idx));
+
+#ifndef QT_NO_DEBUG
+            int existingBindPoint = m_samplerNameMap.value(var.name, -1);
+            Q_ASSERT(existingBindPoint < 0 || existingBindPoint == var.bindPoint);
+#endif
+
             m_samplers.insert(var.bindPoint, vd.value);
             m_samplerNameMap.insert(var.name, var.bindPoint);
         }
@@ -119,7 +139,7 @@ void QSGRhiShaderLinker::dump()
         qDebug() << "Failed to generate program data";
         return;
     }
-    qDebug() << "Combined shader data" << m_vs << m_fs << "cbuffer size" << m_constantBufferSize;
+    qDebug() << "Combined shader data" << m_vs << m_fs;
     qDebug() << " - constants" << m_constants;
     qDebug() << " - samplers" << m_samplers;
 }
@@ -138,40 +158,97 @@ QDebug operator<<(QDebug debug, const QSGRhiShaderLinker::Constant &c)
 
 struct QSGRhiShaderMaterialTypeCache
 {
-    QSGMaterialType *get(const QShader &vs, const QShader &fs);
-    void reset() { qDeleteAll(m_types); m_types.clear(); }
-
+    QSGMaterialType *ref(const QShader &vs, const QShader &fs);
+    void unref(const QShader &vs, const QShader &fs);
+    void reset() {
+        for (auto it = m_types.begin(), end = m_types.end(); it != end; ++it)
+            delete it->type;
+        m_types.clear();
+        clearGraveyard();
+    }
+    void clearGraveyard() {
+        qDeleteAll(m_graveyard);
+        m_graveyard.clear();
+    }
     struct Key {
-        QShader blob[2];
-        Key() { }
-        Key(const QShader &vs, const QShader &fs) { blob[0] = vs; blob[1] = fs; }
+        QShader vs;
+        QShader fs;
+        size_t hash;
+        Key(const QShader &vs, const QShader &fs)
+            : vs(vs),
+              fs(fs)
+        {
+            QtPrivate::QHashCombine hashGen;
+            hash = hashGen(hashGen(0, vs), fs);
+        }
         bool operator==(const Key &other) const {
-            return blob[0] == other.blob[0] && blob[1] == other.blob[1];
+            return vs == other.vs && fs == other.fs;
         }
     };
-    QHash<Key, QSGMaterialType *> m_types;
+    struct MaterialType {
+        int ref;
+        QSGMaterialType *type;
+    };
+    QHash<Key, MaterialType> m_types;
+    QHash<Key, QSGMaterialType *> m_graveyard;
 };
 
 size_t qHash(const QSGRhiShaderMaterialTypeCache::Key &key, size_t seed = 0)
 {
-    size_t hash = seed;
-    for (int i = 0; i < 2; ++i)
-        hash = hash * 31337 + qHash(key.blob[i]);
-    return hash;
+    return seed ^ key.hash;
 }
 
-QSGMaterialType *QSGRhiShaderMaterialTypeCache::get(const QShader &vs, const QShader &fs)
+static QMutex shaderMaterialTypeCacheMutex;
+
+QSGMaterialType *QSGRhiShaderMaterialTypeCache::ref(const QShader &vs, const QShader &fs)
 {
+    QMutexLocker lock(&shaderMaterialTypeCacheMutex);
     const Key k(vs, fs);
-    if (m_types.contains(k))
-        return m_types.value(k);
+    auto it = m_types.find(k);
+    if (it != m_types.end()) {
+        it->ref += 1;
+        return it->type;
+    }
+
+    auto reuseIt = m_graveyard.constFind(k);
+    if (reuseIt != m_graveyard.cend()) {
+        QSGMaterialType *t = reuseIt.value();
+        m_types.insert(k, { 1, t });
+        m_graveyard.erase(reuseIt);
+        return t;
+    }
 
     QSGMaterialType *t = new QSGMaterialType;
-    m_types.insert(k, t);
+    m_types.insert(k, { 1, t });
     return t;
 }
 
-static QSGRhiShaderMaterialTypeCache shaderMaterialTypeCache;
+void QSGRhiShaderMaterialTypeCache::unref(const QShader &vs, const QShader &fs)
+{
+    QMutexLocker lock(&shaderMaterialTypeCacheMutex);
+    const Key k(vs, fs);
+    auto it = m_types.find(k);
+    if (it != m_types.end()) {
+        if (!--it->ref) {
+            m_graveyard.insert(k, it->type);
+            m_types.erase(it);
+        }
+    }
+}
+
+static QHash<void *, QSGRhiShaderMaterialTypeCache> shaderMaterialTypeCache;
+
+void QSGRhiShaderEffectNode::resetMaterialTypeCache(void *materialTypeCacheKey)
+{
+    QMutexLocker lock(&shaderMaterialTypeCacheMutex);
+    shaderMaterialTypeCache[materialTypeCacheKey].reset();
+}
+
+void QSGRhiShaderEffectNode::garbageCollectMaterialTypeCache(void *materialTypeCacheKey)
+{
+    QMutexLocker lock(&shaderMaterialTypeCacheMutex);
+    shaderMaterialTypeCache[materialTypeCacheKey].clearGraveyard();
+}
 
 class QSGRhiShaderEffectMaterialShader : public QSGMaterialShader
 {
@@ -231,8 +308,27 @@ bool QSGRhiShaderEffectMaterialShader::updateUniformData(RenderState &state, QSG
             }
         } else if (c.specialType == QSGShaderEffectNode::VariableData::Matrix) {
             if (state.isMatrixDirty()) {
-                const QMatrix4x4 m = state.combinedMatrix();
-                fillUniformBlockMember<float>(dst, m.constData(), 16, c.size);
+                Q_ASSERT(state.projectionMatrixCount() == mat->viewCount());
+                const int rendererViewCount = state.projectionMatrixCount();
+                const int shaderMatrixCount = c.size / 64;
+                if (shaderMatrixCount < mat->viewCount() && mat->viewCount() >= 2) {
+                    qWarning("qt_Matrix uniform block member size is wrong: expected at least view_count * 64 bytes, "
+                             "where view_count is %d, meaning %d bytes in total, but got only %d bytes. "
+                             "This may be due to the ShaderEffect and its shaders not being multiview-capable, "
+                             "or they are used with an unexpected render target. "
+                             "Check if the shaders declare qt_Matrix as appropriate, "
+                             "and if gl_ViewIndex is used correctly in the vertex shader.",
+                             mat->viewCount(), mat->viewCount() * 64, c.size);
+                }
+                const int matrixCount = qMin(rendererViewCount, shaderMatrixCount);
+                size_t offset = 0;
+                for (int viewIndex = 0; viewIndex < matrixCount; ++viewIndex) {
+                    const QMatrix4x4 m = state.combinedMatrix(viewIndex);
+                    fillUniformBlockMember<float>(dst + offset, m.constData(), 16, 64);
+                    offset += 64;
+                }
+                if (offset < c.size)
+                    memset(dst + offset, 0, c.size - offset);
                 changed = true;
             }
         } else if (c.specialType == QSGShaderEffectNode::VariableData::SubRect) {
@@ -427,6 +523,9 @@ QSGRhiShaderEffectMaterial::QSGRhiShaderEffectMaterial(QSGRhiShaderEffectNode *n
 
 QSGRhiShaderEffectMaterial::~QSGRhiShaderEffectMaterial()
 {
+    if (m_materialType && m_materialTypeCacheKey)
+        shaderMaterialTypeCache[m_materialTypeCacheKey].unref(m_vertexShader, m_fragmentShader);
+
     delete m_dummyTexture;
 }
 
@@ -447,7 +546,7 @@ int QSGRhiShaderEffectMaterial::compare(const QSGMaterial *other) const
     if (int diff = m_cullMode - o->m_cullMode)
         return diff;
 
-    if (int diff = m_textureProviders.count() - o->m_textureProviders.count())
+    if (int diff = m_textureProviders.size() - o->m_textureProviders.size())
         return diff;
 
     if (m_linker.m_constants != o->m_linker.m_constants)
@@ -459,7 +558,7 @@ int QSGRhiShaderEffectMaterial::compare(const QSGMaterial *other) const
     if (hasAtlasTexture(o->m_textureProviders) && !o->m_geometryUsesTextureSubRect)
         return 1;
 
-    for (int binding = 0, count = m_textureProviders.count(); binding != count; ++binding) {
+    for (int binding = 0, count = m_textureProviders.size(); binding != count; ++binding) {
         QSGTextureProvider *tp1 = m_textureProviders.at(binding);
         QSGTextureProvider *tp2 = o->m_textureProviders.at(binding);
         if (tp1 && tp2) {
@@ -559,7 +658,7 @@ QRectF QSGRhiShaderEffectNode::updateNormalizedTextureSubRect(bool supportsAtlas
     bool geometryUsesTextureSubRect = false;
     if (supportsAtlasTextures) {
         QSGTextureProvider *tp = nullptr;
-        for (int binding = 0, count = m_material.m_textureProviders.count(); binding != count; ++binding) {
+        for (int binding = 0, count = m_material.m_textureProviders.size(); binding != count; ++binding) {
             if (QSGTextureProvider *candidate = m_material.m_textureProviders.at(binding)) {
                 if (!tp) {
                     tp = candidate;
@@ -593,10 +692,42 @@ static QShader loadShaderFromFile(const QString &filename)
     return QShader::fromSerialized(f.readAll());
 }
 
+struct QSGRhiShaderEffectDefaultShader
+{
+    QShader shader;
+    quint32 matrixArrayByteSize;
+    quint32 opacityOffset;
+    qint8 viewCount;
+    static QSGRhiShaderEffectDefaultShader create(const QString &filename, int viewCount);
+};
+
+QSGRhiShaderEffectDefaultShader QSGRhiShaderEffectDefaultShader::create(const QString &filename, int viewCount)
+{
+    QSGRhiShaderEffectDefaultShader s;
+    s.shader = loadShaderFromFile(filename);
+    const QList<QShaderDescription::BlockVariable> uboMembers = s.shader.description().uniformBlocks().constFirst().members;
+    for (const auto &member: uboMembers) {
+        if (member.name == QByteArrayLiteral("qt_Matrix"))
+            s.matrixArrayByteSize = member.size;
+        else if (member.name == QByteArrayLiteral("qt_Opacity"))
+            s.opacityOffset = member.offset;
+    }
+    s.viewCount = viewCount;
+    return s;
+}
+
 void QSGRhiShaderEffectNode::syncMaterial(SyncData *syncData)
 {
-    static QShader defaultVertexShader;
-    static QShader defaultFragmentShader;
+    static const int defaultVertexShaderCount = 2;
+    static QSGRhiShaderEffectDefaultShader defaultVertexShaders[defaultVertexShaderCount] = {
+        QSGRhiShaderEffectDefaultShader::create(QStringLiteral(":/qt-project.org/scenegraph/shaders_ng/shadereffect.vert.qsb"), 1),
+        QSGRhiShaderEffectDefaultShader::create(QStringLiteral(":/qt-project.org/scenegraph/shaders_ng/shadereffect.vert.qsb.mv2qsb"), 2)
+    };
+    static const int defaultFragmentShaderCount = 2;
+    static QSGRhiShaderEffectDefaultShader defaultFragmentShaders[defaultFragmentShaderCount] = {
+        QSGRhiShaderEffectDefaultShader::create(QStringLiteral(":/qt-project.org/scenegraph/shaders_ng/shadereffect.frag.qsb"), 1),
+        QSGRhiShaderEffectDefaultShader::create(QStringLiteral(":/qt-project.org/scenegraph/shaders_ng/shadereffect.frag.qsb.mv2qsb"), 2)
+    };
 
     if (bool(m_material.flags() & QSGMaterial::Blending) != syncData->blending) {
         m_material.setFlag(QSGMaterial::Blending, syncData->blending);
@@ -609,25 +740,57 @@ void QSGRhiShaderEffectNode::syncMaterial(SyncData *syncData)
     }
 
     if (syncData->dirty & QSGShaderEffectNode::DirtyShaders) {
+        if (m_material.m_materialType) {
+            shaderMaterialTypeCache[m_material.m_materialTypeCacheKey].unref(m_material.m_vertexShader,
+                                                                             m_material.m_fragmentShader);
+        }
+
         m_material.m_hasCustomVertexShader = syncData->vertex.shader->hasShaderCode;
+        quint32 defaultMatrixArrayByteSize = 0;
         if (m_material.m_hasCustomVertexShader) {
             m_material.m_vertexShader = syncData->vertex.shader->shaderInfo.rhiShader;
         } else {
-            if (!defaultVertexShader.isValid())
-                defaultVertexShader = loadShaderFromFile(QStringLiteral(":/qt-project.org/scenegraph/shaders_ng/shadereffect.vert.qsb"));
-            m_material.m_vertexShader = defaultVertexShader;
+            bool found = false;
+            for (int i = 0; i < defaultVertexShaderCount; ++i) {
+                if (defaultVertexShaders[i].viewCount == syncData->viewCount) {
+                    m_material.m_vertexShader = defaultVertexShaders[i].shader;
+                    defaultMatrixArrayByteSize = defaultVertexShaders[i].matrixArrayByteSize;
+                    found = true;
+                    break;
+                }
+            }
+            if (!found) {
+                qWarning("No default vertex shader found for view count %d", syncData->viewCount);
+                m_material.m_vertexShader = defaultVertexShaders[0].shader;
+                defaultMatrixArrayByteSize = 64;
+            }
         }
 
         m_material.m_hasCustomFragmentShader = syncData->fragment.shader->hasShaderCode;
+        quint32 defaultOpacityOffset = 0;
         if (m_material.m_hasCustomFragmentShader) {
             m_material.m_fragmentShader = syncData->fragment.shader->shaderInfo.rhiShader;
         } else {
-            if (!defaultFragmentShader.isValid())
-                defaultFragmentShader = loadShaderFromFile(QStringLiteral(":/qt-project.org/scenegraph/shaders_ng/shadereffect.frag.qsb"));
-            m_material.m_fragmentShader = defaultFragmentShader;
+            bool found = false;
+            for (int i = 0; i < defaultFragmentShaderCount; ++i) {
+                if (defaultFragmentShaders[i].viewCount == syncData->viewCount) {
+                    m_material.m_fragmentShader = defaultFragmentShaders[i].shader;
+                    defaultOpacityOffset = defaultFragmentShaders[i].opacityOffset;
+                    found = true;
+                    break;
+                }
+            }
+            if (!found) {
+                qWarning("No default fragment shader found for view count %d", syncData->viewCount);
+                m_material.m_fragmentShader = defaultFragmentShaders[0].shader;
+                defaultOpacityOffset = 64;
+            }
         }
 
-        m_material.m_materialType = shaderMaterialTypeCache.get(m_material.m_vertexShader, m_material.m_fragmentShader);
+        m_material.m_materialType = shaderMaterialTypeCache[syncData->materialTypeCacheKey].ref(m_material.m_vertexShader,
+                                                                                                m_material.m_fragmentShader);
+        m_material.m_materialTypeCacheKey = syncData->materialTypeCacheKey;
+
         m_material.m_linker.reset(m_material.m_vertexShader, m_material.m_fragmentShader);
 
         if (m_material.m_hasCustomVertexShader) {
@@ -639,16 +802,15 @@ void QSGRhiShaderEffectNode::syncMaterial(SyncData *syncData)
             defaultSD.shaderInfo.rhiShader = m_material.m_vertexShader;
             defaultSD.shaderInfo.type = QSGGuiThreadShaderEffectManager::ShaderInfo::TypeVertex;
 
-            // { mat4 qt_Matrix; float qt_Opacity; } where only the matrix is used
+            // { mat4 qt_Matrix[VIEW_COUNT]; float qt_Opacity; } where only the matrix is used
             QSGGuiThreadShaderEffectManager::ShaderInfo::Variable v;
             v.name = QByteArrayLiteral("qt_Matrix");
             v.offset = 0;
-            v.size = 16 * sizeof(float);
+            v.size = defaultMatrixArrayByteSize;
             defaultSD.shaderInfo.variables.append(v);
             QSGShaderEffectNode::VariableData vd;
             vd.specialType = QSGShaderEffectNode::VariableData::Matrix;
             defaultSD.varData.append(vd);
-            defaultSD.shaderInfo.constantDataSize = (16 + 1) * sizeof(float);
             m_material.m_linker.feedConstants(defaultSD);
         }
 
@@ -661,10 +823,10 @@ void QSGRhiShaderEffectNode::syncMaterial(SyncData *syncData)
             defaultSD.shaderInfo.rhiShader = m_material.m_fragmentShader;
             defaultSD.shaderInfo.type = QSGGuiThreadShaderEffectManager::ShaderInfo::TypeFragment;
 
-            // { mat4 qt_Matrix; float qt_Opacity; } where only the opacity is used
+            // { mat4 qt_Matrix[VIEW_COUNT]; float qt_Opacity; } where only the opacity is used
             QSGGuiThreadShaderEffectManager::ShaderInfo::Variable v;
             v.name = QByteArrayLiteral("qt_Opacity");
-            v.offset = 16 * sizeof(float);
+            v.offset = defaultOpacityOffset;
             v.size = sizeof(float);
             defaultSD.shaderInfo.variables.append(v);
             QSGShaderEffectNode::VariableData vd;
@@ -675,7 +837,7 @@ void QSGRhiShaderEffectNode::syncMaterial(SyncData *syncData)
             v.bindPoint = 1;
             v.type = QSGGuiThreadShaderEffectManager::ShaderInfo::Sampler;
             defaultSD.shaderInfo.variables.append(v);
-            for (const QSGShaderEffectNode::VariableData &extVarData : qAsConst(syncData->fragment.shader->varData)) {
+            for (const QSGShaderEffectNode::VariableData &extVarData : std::as_const(syncData->fragment.shader->varData)) {
                 if (extVarData.specialType == QSGShaderEffectNode::VariableData::Source) {
                     vd.value = extVarData.value;
                     break;
@@ -684,8 +846,6 @@ void QSGRhiShaderEffectNode::syncMaterial(SyncData *syncData)
             vd.specialType = QSGShaderEffectNode::VariableData::Source;
             defaultSD.varData.append(vd);
 
-            defaultSD.shaderInfo.constantDataSize = (16 + 1) * sizeof(float);
-
             m_material.m_linker.feedConstants(defaultSD);
             m_material.m_linker.feedSamplers(defaultSD);
         }
@@ -745,11 +905,6 @@ void QSGRhiShaderEffectNode::preprocess()
     }
 }
 
-void QSGRhiShaderEffectNode::cleanupMaterialTypeCache()
-{
-    shaderMaterialTypeCache.reset();
-}
-
 bool QSGRhiGuiThreadShaderEffectManager::hasSeparateSamplerAndTextureObjects() const
 {
     return false; // because SPIR-V and QRhi make it look so, regardless of the underlying API
@@ -814,16 +969,14 @@ bool QSGRhiGuiThreadShaderEffectManager::reflect(ShaderInfo *result)
     }
 
     const QShaderDescription desc = result->rhiShader.description();
-    result->constantDataSize = 0;
 
     int ubufBinding = -1;
     const QVector<QShaderDescription::UniformBlock> ubufs = desc.uniformBlocks();
-    const int ubufCount = ubufs.count();
+    const int ubufCount = ubufs.size();
     for (int i = 0; i < ubufCount; ++i) {
         const QShaderDescription::UniformBlock &ubuf(ubufs[i]);
         if (ubufBinding == -1 && ubuf.binding >= 0) {
             ubufBinding = ubuf.binding;
-            result->constantDataSize = ubuf.size;
             for (const QShaderDescription::BlockVariable &member : ubuf.members) {
                 ShaderInfo::Variable v;
                 v.type = ShaderInfo::Constant;
@@ -839,7 +992,7 @@ bool QSGRhiGuiThreadShaderEffectManager::reflect(ShaderInfo *result)
     }
 
     const QVector<QShaderDescription::InOutVariable> combinedImageSamplers = desc.combinedImageSamplers();
-    const int samplerCount = combinedImageSamplers.count();
+    const int samplerCount = combinedImageSamplers.size();
     for (int i = 0; i < samplerCount; ++i) {
         const QShaderDescription::InOutVariable &combinedImageSampler(combinedImageSamplers[i]);
         ShaderInfo::Variable v;
