@@ -357,8 +357,8 @@ static bool hasGlobalBackground = false;
 // values that QColor accepts, as opposed to one of the pre-defined colors like Red.
 static bool globalPrimaryCustom = false;
 static bool globalAccentCustom = false;
-static bool globalForegroundCustom = true;
-static bool globalBackgroundCustom = true;
+static bool globalForegroundCustom = false;
+static bool globalBackgroundCustom = false;
 // This is global because:
 // 1) The theme needs access to it to determine font sizes.
 // 2) There can only be one variant used for the whole application.
@@ -490,9 +490,9 @@ void QQuickMaterialStyle::setTheme(Theme theme)
     themeChange();
     if (!m_customAccent)
         accentChange();
-    if (!m_hasBackground)
+    if (!m_customBackground)
         backgroundChange();
-    if (!m_hasForeground)
+    if (!m_customForeground)
         foregroundChange();
 }
 
@@ -510,9 +510,9 @@ void QQuickMaterialStyle::inheritTheme(Theme theme)
     themeChange();
     if (!m_customAccent)
         accentChange();
-    if (!m_hasBackground)
+    if (!m_customBackground)
         backgroundChange();
-    if (!m_hasForeground)
+    if (!m_customForeground)
         foregroundChange();
 }
 
@@ -679,8 +679,12 @@ void QQuickMaterialStyle::accentChange()
 
 QVariant QQuickMaterialStyle::foreground() const
 {
-    if (!m_hasForeground)
-        return QColor::fromRgba(m_theme == Light ? primaryTextColorLight : primaryTextColorDark);
+    if (!m_hasForeground) {
+        if (!m_customBackground && m_background == m_primary) {
+            return toolTextColor();
+        }
+         return QColor::fromRgba(m_theme == Light ? primaryTextColorLight : primaryTextColorDark);
+    }
     if (m_customForeground)
         return QColor::fromRgba(m_foreground);
     if (m_foreground > BlueGrey)
@@ -695,6 +699,16 @@ void QQuickMaterialStyle::setForeground(const QVariant &var)
     if (!variantToRgba(var, "foreground", &foreground, &custom))
         return;
 
+    // Reset the foreground if the new color matches the current theme's default.
+    // This allows the color to update dynamically when the theme changes.
+    if (!hasGlobalForeground) {
+        const QRgb themeDefault = (m_theme == Light ? primaryTextColorLight : primaryTextColorDark);
+        if (foreground == themeDefault) {
+            resetForeground();
+            return;
+        }
+    }
+
     m_hasForeground = true;
     m_explicitForeground = true;
     if (m_foreground == foreground)
@@ -744,8 +758,6 @@ void QQuickMaterialStyle::foregroundChange()
 {
     emit foregroundChanged();
     emit primaryHighlightedTextColorChanged();
-    // TODO: This causes a binding loop: see QTBUG-85699 and the comments on its fix
-//    emit toolTextColorChanged();
 }
 
 QVariant QQuickMaterialStyle::background() const
@@ -760,6 +772,24 @@ void QQuickMaterialStyle::setBackground(const QVariant &var)
     if (!variantToRgba(var, "background", &background, &custom))
         return;
 
+    // Reset the background if the new color matches the current theme's default.
+    // This allows the color to update dynamically when the theme changes.
+    if (!hasGlobalBackground) {
+        const QRgb themeDefault = (m_theme == Light ? backgroundColorLight : backgroundColorDark);
+        if (background == themeDefault) {
+            resetBackground();
+            return;
+        }
+    }
+
+    // Do not treat the background as custom if it resolves to the same RGB as the
+    // current primary color. This allows the foreground color to map to
+    // toolTextColor, which is resolved dynamically based on the primary color.
+    if (custom && background == primaryColor().rgb()) {
+        background = m_primary;
+        custom = false;
+    }
+
     m_hasBackground = true;
     m_explicitBackground = true;
     if (m_background == background)
@@ -769,6 +799,10 @@ void QQuickMaterialStyle::setBackground(const QVariant &var)
     m_background = background;
     propagateBackground();
     backgroundChange();
+
+    // Foreground color depends on the background color if the background is set to primary.
+    if (!m_customBackground && m_background == m_primary)
+        foregroundChange();
 }
 
 void QQuickMaterialStyle::inheritBackground(uint background, bool custom, bool has)
@@ -781,6 +815,8 @@ void QQuickMaterialStyle::inheritBackground(uint background, bool custom, bool h
     m_background = background;
     propagateBackground();
     backgroundChange();
+    if (!m_customBackground && m_background == m_primary)
+        foregroundChange();
 }
 
 void QQuickMaterialStyle::propagateBackground()
@@ -903,7 +939,9 @@ QColor QQuickMaterialStyle::backgroundColor(Shade shade) const
 {
     if (!m_hasBackground)
         return QColor::fromRgba(m_theme == Light ? backgroundColorLight : backgroundColorDark);
-    if (m_customBackground)
+    // The application bars (such as ToolBar) set the background color from the primary color,
+    // and in that case, it's better to validate with the custom primary and use it accordingly.
+    if (m_customBackground || (m_customPrimary && m_background == m_primary))
         return shade == themeShade() ? QColor::fromRgba(m_background)
                                      : this->shade(QColor::fromRgba(m_background), shade);
     if (m_background > BlueGrey)
