@@ -2,23 +2,21 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-83gs
+es5id: 15.3.5.4_2-85gs
 description: >
     Strict mode - checking access to strict function caller from
     non-strict function (non-strict function declaration called by
-    strict Function.prototype.apply(globalObject))
+    strict Function.prototype.call(null))
 flags: [noStrict]
 features: [caller]
 ---*/
 
-var global = this;
-
 function f() {
   return gNonStrict();
 };
 (function() {
   "use strict";
-  f.apply(global);
+  f.call(null);
 })();
 
 
