@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.7-5-b-40
+es5id: 15.2.3.7-5-b-33
 description: >
     Object.defineProperties - value of 'enumerable' property of
-    'descObj' is positive number (8.10.5 step 3.b)
+    'descObj' is null (8.10.5 step 3.b)
 ---*/
 
 var obj = {};
@@ -13,7 +13,7 @@ var accessed = false;
 
 Object.defineProperties(obj, {
   prop: {
-    enumerable: 12
+    enumerable: null
   }
 });
 for (var property in obj) {
@@ -22,4 +22,4 @@ for (var property in obj) {
   }
 }
 
-assert(accessed, 'accessed !== true');
+assert.sameValue(accessed, false, 'accessed');
