@@ -2,10 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.11-4-8
-description: Object.isSealed returns false for all built-in objects (String)
+es5id: 15.2.3.11-4-24
+description: >
+    Object.isSealed returns false for all built-in objects
+    (SyntaxError)
 ---*/
 
-var b = Object.isSealed(String);
+var b = Object.isSealed(SyntaxError);
 
 assert.sameValue(b, false, 'b');
