@@ -5,12 +5,12 @@
 info: |
     15.5.5.2 defines [[GetOwnProperty]] for Strings. It supports using indexing
     notation to look up non numeric property names.
-es5id: 15.5.5.5.2-7-4
+es5id: 15.5.5.5.2-3-5
 description: >
-    String value indexing returns undefined if the numeric index is
-    greater than the string length
+    String object indexing returns undefined if the numeric index (
+    2^32-1) is not an array index
 ---*/
 
-var s = String("hello world");
+var s = new String("hello world");
 
-assert.sameValue(s[11], undefined, 's[11]');
+assert.sameValue(s[Math.pow(2, 32) - 1], undefined, 's[Math.pow(2, 32)-1]');
