@@ -2,18 +2,17 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 11.4.1-5-1
-description: >
-    delete operator returns false when deleting a direct reference to
-    a var
+info: |
+    This test is actually testing the [[Delete]] internal method (8.12.8). Since the
+    language provides no way to directly exercise [[Delete]], the tests are placed here.
+es5id: 11.4.1-4.a-7
+description: delete operator inside 'eval'
 flags: [noStrict]
 ---*/
 
 function testcase() {
   var x = 1;
-
-  // Now, deleting 'x' directly should fail;
-  var d = delete x;
+  var d = eval("delete x");
 
   assert.sameValue(d, false, 'd');
   assert.sameValue(x, 1, 'x');
