@@ -2,19 +2,20 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 10.4.3-1-27gs
+es5id: 10.4.3-1-43gs
 description: >
-    Strict - checking 'this' from a global scope (FunctionDeclaration
-    defined within a FunctionDeclaration inside strict mode)
-flags: [onlyStrict]
+    Strict - checking 'this' from a global scope (FunctionExpression
+    defined within an Anonymous FunctionExpression with a strict
+    directive prologue)
+flags: [noStrict]
 ---*/
 
-function f1() {
-    function f() {
+if (! ((function () {
+    "use strict";
+    var f = function () {
         return typeof this;
     }
     return (f()==="undefined") && ((typeof this)==="undefined");
-}
-if (! f1()) {
+})())) {
     throw "'this' had incorrect value!";
 }
