@@ -2,15 +2,17 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: NonEscapeSequence is not EscapeCharacter
-es5id: 7.8.4_A4.3_T1
-description: "EscapeCharacter :: DecimalDigits :: 1"
+info: "StringLiteral :: 'SingleStringCharacters_opt'"
+es5id: 7.8.4_A1.2_T1
+description: >
+    SingleStringCharacter :: SourceCharacter but not single-quote ' or
+    LineTerminator
 negative:
   phase: parse
   type: SyntaxError
-flags: [onlyStrict]
 ---*/
 
 throw "Test262: This statement should not be evaluated.";
 
-"\1"
+//CHECK#1
+'''
