@@ -915,15 +915,23 @@ class MyCustomParserType : public QObject
 class MyCustomParserTypeParser : public QQmlCustomParser
 {
 public:
-    void verifyBindings(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override {}
-    void applyBindings(QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override {}
+    void verifyBindings(
+            const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override {}
+    void applyBindings(
+            QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override {}
 };
 
 class EnumSupportingCustomParser : public QQmlCustomParser
 {
 public:
-    void verifyBindings(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override;
-    void applyBindings(QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override {}
+    void verifyBindings(
+            const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override;
+    void applyBindings(
+            QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override {}
 };
 
 class MyParserStatus : public QObject, public QQmlParserStatus
@@ -1450,8 +1458,12 @@ public:
 
 class CustomBindingParser : public QQmlCustomParser
 {
-    void verifyBindings(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override {}
-    void applyBindings(QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override;
+    void verifyBindings(
+            const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override {}
+    void applyBindings(
+            QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override;
 };
 
 class SimpleObjectWithCustomParser : public QObject
@@ -1496,8 +1508,12 @@ private:
 
 class SimpleObjectCustomParser : public QQmlCustomParser
 {
-    void verifyBindings(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override {}
-    void applyBindings(QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &, const QList<const QV4::CompiledData::Binding *> &) override;
+    void verifyBindings(
+            const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override {}
+    void applyBindings(
+            QObject *, const QQmlRefPointer<QV4::ExecutableCompilationUnit> &,
+            const QList<const QV4::CompiledData::Binding *> &) override;
 };
 
 class RootObjectInCreationTester : public QObject
@@ -3058,4 +3074,209 @@ private:
     std::vector<std::vector<int>> m_list;
 };
 
+class VariantAssociationProvider : public QObject {
+
+    Q_OBJECT
+    Q_PROPERTY(QVariantMap variantMap READ getMap WRITE setMap)
+    Q_PROPERTY(QVariantHash variantHash READ getHash WRITE setHash)
+    Q_PROPERTY(QVariantList variantList READ getList WRITE setList)
+
+    QML_ELEMENT
+
+    QVariantMap m_variantMap;
+    QVariantHash m_variantHash;
+    QVariantList m_variantList;
+
+public:
+    VariantAssociationProvider(QObject* parent = nullptr) : QObject(parent)
+    {}
+
+    QVariantMap getMap() { return m_variantMap; }
+    void setMap(const QVariantMap& map) { m_variantMap = map; }
+
+    QVariantHash getHash() { return m_variantHash; }
+    void setHash(const QVariantHash& hash) { m_variantHash = hash; }
+
+    QVariantList getList() { return m_variantList; }
+    void setList(const QVariantList &list) { m_variantList = list; }
+
+    Q_INVOKABLE QVariantList getListOfMap() const {
+        return QVariantList{
+            QVariantMap{},
+            QVariantMap{ { "email", "Alice Smith"} }
+        };
+    }
+
+    Q_INVOKABLE QVariantList getListOfHash() const {
+        return QVariantList{
+            QVariantHash{},
+            QVariantHash{ { "email", "Alice Smith"} }
+        };
+    }
+};
+
+namespace YepNamespaceA {
+class YepAttached : public QObject
+{
+    Q_OBJECT
+    QML_ANONYMOUS
+
+public:
+    YepAttached(QObject *parent) : QObject(parent) { }
+    Q_INVOKABLE QString s() const { return QStringLiteral("StaticTest Attached Type"); }
+};
+class Yep : public QObject
+{
+    Q_OBJECT
+    QML_ATTACHED(YepAttached)
+    QML_ELEMENT
+
+public:
+    static YepAttached *qmlAttachedProperties(QObject *object) { return new YepAttached(object); }
+};
+
+class YepSingleton : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+
+public:
+    Q_INVOKABLE QString s() const { return QStringLiteral("StaticTest Singleton"); }
+};
+
+class MyObject : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    Q_INVOKABLE QString s() const { return QStringLiteral("StaticTest"); }
+};
+} // namespace YepNamespaceA
+
+class BindablePoint : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(QPointF point BINDABLE bindablePoint READ default WRITE default)
+public:
+    BindablePoint(QObject *parent = nullptr) : QObject(parent), m_point(QPointF(101, 102)) {}
+
+    QBindable<QPointF> bindablePoint() { return QBindable<QPointF>(&m_point); }
+
+private:
+    QProperty<QPointF> m_point;
+};
+
+class LargeRevisionBase : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(int c MEMBER m_c CONSTANT)
+    Q_PROPERTY(int d MEMBER m_d CONSTANT)
+public:
+    int m_c = 11;
+    int m_d = 12;
+};
+
+class LargeRevision : public LargeRevisionBase
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(int a MEMBER m_a CONSTANT REVISION(1, 12))
+    Q_PROPERTY(int b MEMBER m_b CONSTANT REVISION(12, 12))
+    Q_PROPERTY(int c MEMBER m_c CONSTANT REVISION(1, 12))
+    Q_PROPERTY(int d MEMBER m_d CONSTANT REVISION(12, 12))
+public:
+    int m_a = 13;
+    int m_b = 14;
+    int m_c = 15;
+    int m_d = 16;
+};
+
+class CppEnum : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    enum class Scoped { S1, S2, S3 = 5, S4 = 5 };
+    Q_ENUM(Scoped)
+    enum Unscoped { U1, U2, U3 = 5, U4 = 5 };
+    Q_ENUM(Unscoped)
+};
+
+namespace EnumNamespace {
+Q_NAMESPACE
+QML_ELEMENT
+
+enum Unscoped { U1, U2, U3 = 5, U4 = 5 };
+Q_ENUM_NS(Unscoped)
+enum class Scoped { S1, S2, S3 = 5, S4 = 5 };
+Q_ENUM_NS(Scoped)
+} // namespace EnumNamespace
+
+class ConflictingEnums : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    enum class E1 { A = 1 };
+    Q_ENUM(E1);
+    enum E2 { A = 2 };
+    Q_ENUM(E2);
+};
+
+template <class T>
+class IdType
+{
+public:
+    IdType(T id) : value(id) {}
+    T get() const { return value; }
+
+private:
+    T value;
+};
+
+using CustomId = IdType<int32_t>;
+
+class CustomIdentifier
+{
+    Q_GADGET
+    QML_VALUE_TYPE(customIdentifier)
+    QML_STRUCTURED_VALUE
+
+    Q_PROPERTY(int customId READ getCustomId CONSTANT FINAL)
+public:
+    CustomIdentifier() = default;
+    CustomIdentifier(CustomId custom_id) : id(std::move(custom_id)) {}
+
+    int getCustomId() const { return id.get(); }
+    Q_INVOKABLE int toVisualId() const { return id.get() + 1; }
+
+    friend bool operator==(const CustomIdentifier &a, const CustomIdentifier &b)
+    {
+        return a.id.get() == b.id.get();
+    }
+
+    CustomId id {-1};
+};
+
+class IdProvider : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+public:
+    IdProvider() = default;
+
+    static CustomId id1() { return CustomId(20); }
+    static CustomId id2() { return CustomId(21); }
+
+    Q_INVOKABLE CustomIdentifier getId1() const { return CustomIdentifier(id1()); }
+    Q_INVOKABLE CustomIdentifier getId2() const { return CustomIdentifier(id2()); }
+};
+
 #endif // TESTTYPES_H
