@@ -23,6 +23,8 @@
 
 #include <memory>
 
+#include <private/qqmlobjectcreator_p.h>
+
 class tst_qv4mm : public QQmlDataTest
 {
     Q_OBJECT
@@ -46,7 +48,17 @@ private slots:
     void jittedStoreLocalMarksValue();
     void forInOnProxyMarksTarget();
     void allocWithMemberDataMidwayDrain();
+    void constObjectWrapperOnlyConstInSingleEngine();
     void markObjectWrappersAfterMarkWeakValues();
+
+    void trackObjectDoesNotAccessGarbageOnTheStackOnAllocation();
+    void spreadArgumentDoesNotAccessGarbageOnTheStackOnAllocation();
+    void scopedConvertToStringFromReturnedValueDoesNotAccessGarbageOnTheStackOnAllocation();
+    void scopedConvertToObjectFromReturnedValueDoesNotAccessGarbageOnTheStackOnAllocation();
+    void scopedConvertToStringFromValueDoesNotAccessGarbageOnTheStackOnAllocation();
+    void scopedConvertToObjectFromValueDoesNotAccessGarbageOnTheStackOnAllocation();
+
+    void dontCrashOnScopedStackFrame();
 };
 
 tst_qv4mm::tst_qv4mm()
@@ -682,6 +694,9 @@ QV4::ReturnedValue method_force_jit(const QV4::FunctionObject *b, const QV4::Val
     QV4::JIT::BaselineJIT(func).generate();
     return QV4::StaticValue::fromBoolean(true).asReturnedValue();
 #else
+    Q_UNUSED(b);
+    Q_UNUSED(argv);
+    Q_UNUSED(argc);
     return QV4::StaticValue::fromBoolean(false).asReturnedValue();
 #endif
 }
@@ -797,6 +812,32 @@ void tst_qv4mm::allocWithMemberDataMidwayDrain()
     QVERIFY(o); // dummy check
 }
 
+void tst_qv4mm::constObjectWrapperOnlyConstInSingleEngine()
+{
+    auto myObject = new QObject(this);
+    QV4::ExecutionEngine e1 {};
+    // prevent the gc from running too early
+    e1.memoryManager->gcBlocked = QV4::MemoryManager::NormalBlocked;
+    // create a non-const wrapper in the first engine
+    QV4::QObjectWrapper::ensureWrapper(&e1, myObject);
+    {
+        QV4::ExecutionEngine e2 {};
+        QV4::Scope scope(&e2);
+        // create a const wrapper in the second engine
+        QV4::ScopedValue val(scope, QV4::QObjectWrapper::wrapConst(&e2, myObject));
+    }
+    QVERIFY(!e1.m_multiplyWrappedQObjects);
+    auto ddata = QQmlData::get(myObject, false);
+    QVERIFY(ddata);
+    QVERIFY(ddata->hasConstWrapper);
+    e1.memoryManager->gcBlocked = QV4::MemoryManager::Unblocked;
+    gc(e1); // should not trigger an assert
+    // hasConstWrapper only tells us that at some point, some engine created one;
+    // the const wrappers are not actually shared, but we can't know when the last
+    // one is gone
+    QVERIFY(ddata->hasConstWrapper);
+}
+
 void tst_qv4mm::markObjectWrappersAfterMarkWeakValues()
 {
     // Advance gc to just after MarkWeakValues
@@ -832,6 +873,165 @@ void tst_qv4mm::markObjectWrappersAfterMarkWeakValues()
     QCOMPARE(qvariant_cast<QObject *>(retrieved)->objectName(), "yep");
 }
 
+void tst_qv4mm::trackObjectDoesNotAccessGarbageOnTheStackOnAllocation()
+{
+#if defined(QT_NO_DEBUG) && !defined(QT_FORCE_ASSERTS)
+    QSKIP("Our main visibility tool for this test is an assertion. Thus the test needs assertions to be enabled.");
+#endif
+
+    qputenv("QV4_MM_AGGRESSIVE_GC", "1");
+    QScopeGuard env_guard([](){ qunsetenv("QV4_MM_AGGRESSIVE_GC"); });
+
+    qputenv("QV4_GC_TIMELIMIT", "0");
+    QScopeGuard env_guard2([](){ qunsetenv("QV4_GC_TIMELIMIT"); });
+
+    QJSEngine jsengine;
+    QV4::ExecutionEngine* engine = jsengine.handle();
+    *engine->jsStackTop = QV4::Value::fromHeapObject(engine->memoryManager->allocManaged<QV4::String>());
+
+    jsengine.collectGarbage();
+
+    QV4::Scope scope(engine);
+    ObjectInCreationGCAnchorList tracker(scope, 1);
+
+    QObject object{};
+    tracker.trackObject(engine, &object);
+}
+
+void tst_qv4mm::spreadArgumentDoesNotAccessGarbageOnTheStackOnAllocation()
+{
+#if defined(QT_NO_DEBUG) && !defined(QT_FORCE_ASSERTS)
+    QSKIP("Our main visibility tool for this test is an assertion. Thus the test needs assertions to be enabled.");
+#endif
+
+    qputenv("QV4_MM_AGGRESSIVE_GC", "1");
+    QScopeGuard env_guard([](){ qunsetenv("QV4_MM_AGGRESSIVE_GC"); });
+
+    qputenv("QV4_GC_TIMELIMIT", "0");
+    QScopeGuard env_guard2([](){ qunsetenv("QV4_GC_TIMELIMIT"); });
+
+    QJSEngine jsengine;
+    QV4::ExecutionEngine* engine = jsengine.handle();
+
+    QJSValue function = jsengine.evaluate("function foo(){}; foo");
+    QV4::Value* function_value = QJSValuePrivate::takeManagedValue(&function);
+
+    QV4::Scope scope(engine);
+    QV4::ScopedObject thisobject(scope);
+    QV4::ScopedString spread(scope, engine->newString(QString(u"abc")));
+    QV4::Value argv[2] = { QV4::Value::emptyValue(), spread };
+
+    {
+        QV4::Value value = QV4::Value::fromHeapObject(engine->memoryManager->allocManaged<QV4::String>());
+        for (QV4::Value* top = engine->jsStackTop; top < engine->jsStackLimit; ++top)
+            *top = value;
+    }
+
+    jsengine.collectGarbage();
+
+    QV4::Runtime::CallWithSpread::call(engine, *function_value, thisobject, argv, 1);
+}
+
+void tst_qv4mm::scopedConvertToStringFromReturnedValueDoesNotAccessGarbageOnTheStackOnAllocation()
+{
+#if defined(QT_NO_DEBUG) && !defined(QT_FORCE_ASSERTS)
+    QSKIP("Our main visibility tool for this test is an assertion. Thus the test needs assertions to be enabled.");
+#endif
+
+    qputenv("QV4_MM_AGGRESSIVE_GC", "1");
+    QScopeGuard env_guard([](){ qunsetenv("QV4_MM_AGGRESSIVE_GC"); });
+
+    qputenv("QV4_GC_TIMELIMIT", "0");
+    QScopeGuard env_guard2([](){ qunsetenv("QV4_GC_TIMELIMIT"); });
+
+    QJSEngine jsengine;
+    QV4::ExecutionEngine* engine = jsengine.handle();
+    *engine->jsStackTop = QV4::Value::fromHeapObject(engine->memoryManager->allocManaged<QV4::String>());
+
+    jsengine.collectGarbage();
+
+    QV4::Scope scope(engine);
+    QV4::ScopedString string(scope, QV4::StaticValue::fromDouble(1.24).asReturnedValue(), QV4::ScopedString::Convert);
+}
+
+void tst_qv4mm::scopedConvertToObjectFromReturnedValueDoesNotAccessGarbageOnTheStackOnAllocation()
+{
+#if defined(QT_NO_DEBUG) && !defined(QT_FORCE_ASSERTS)
+    QSKIP("Our main visibility tool for this test is an assertion. Thus the test needs assertions to be enabled.");
+#endif
+
+    qputenv("QV4_MM_AGGRESSIVE_GC", "1");
+    QScopeGuard env_guard([](){ qunsetenv("QV4_MM_AGGRESSIVE_GC"); });
+
+    qputenv("QV4_GC_TIMELIMIT", "0");
+    QScopeGuard env_guard2([](){ qunsetenv("QV4_GC_TIMELIMIT"); });
+
+    QJSEngine jsengine;
+    QV4::ExecutionEngine* engine = jsengine.handle();
+    *engine->jsStackTop = QV4::Value::fromHeapObject(engine->memoryManager->allocManaged<QV4::String>());
+
+    jsengine.collectGarbage();
+
+    QV4::Scope scope(engine);
+    QV4::ScopedObject object(scope, QV4::StaticValue::fromBoolean(true).asReturnedValue(), QV4::ScopedObject::Convert);
+}
+
+void tst_qv4mm::scopedConvertToStringFromValueDoesNotAccessGarbageOnTheStackOnAllocation()
+{
+#if defined(QT_NO_DEBUG) && !defined(QT_FORCE_ASSERTS)
+    QSKIP("Our main visibility tool for this test is an assertion. Thus the test needs assertions to be enabled.");
+#endif
+
+    qputenv("QV4_MM_AGGRESSIVE_GC", "1");
+    QScopeGuard env_guard([](){ qunsetenv("QV4_MM_AGGRESSIVE_GC"); });
+
+    qputenv("QV4_GC_TIMELIMIT", "0");
+    QScopeGuard env_guard2([](){ qunsetenv("QV4_GC_TIMELIMIT"); });
+
+    QJSEngine jsengine;
+    QV4::ExecutionEngine* engine = jsengine.handle();
+    *engine->jsStackTop = QV4::Value::fromHeapObject(engine->memoryManager->allocManaged<QV4::String>());
+
+    jsengine.collectGarbage();
+
+    QV4::Scope scope(engine);
+    QV4::ScopedString string(scope, QV4::StaticValue::fromDouble(1.24).asValue<QV4::Value>(), QV4::ScopedString::Convert);
+}
+
+
+void tst_qv4mm::scopedConvertToObjectFromValueDoesNotAccessGarbageOnTheStackOnAllocation()
+{
+#if defined(QT_NO_DEBUG) && !defined(QT_FORCE_ASSERTS)
+    QSKIP("Our main visibility tool for this test is an assertion. Thus the test needs assertions to be enabled.");
+#endif
+
+    qputenv("QV4_MM_AGGRESSIVE_GC", "1");
+    QScopeGuard env_guard([](){ qunsetenv("QV4_MM_AGGRESSIVE_GC"); });
+
+    qputenv("QV4_GC_TIMELIMIT", "0");
+    QScopeGuard env_guard2([](){ qunsetenv("QV4_GC_TIMELIMIT"); });
+
+    QJSEngine jsengine;
+    QV4::ExecutionEngine* engine = jsengine.handle();
+    *engine->jsStackTop = QV4::Value::fromHeapObject(engine->memoryManager->allocManaged<QV4::String>());
+
+    jsengine.collectGarbage();
+
+    QV4::Scope scope(engine);
+    QV4::ScopedObject object(scope, QV4::StaticValue::fromBoolean(true).asValue<QV4::Value>(), QV4::ScopedObject::Convert);
+}
+
+void tst_qv4mm::dontCrashOnScopedStackFrame()
+{
+    QJSEngine jsengine;
+    QV4::ExecutionEngine *engine = jsengine.handle();
+
+    QV4::Scope scope(engine);
+    QV4::ScopedStackFrame frame(scope, engine->rootContext());
+
+    jsengine.collectGarbage();
+}
+
 QTEST_MAIN(tst_qv4mm)
 
 #include "tst_qv4mm.moc"
