@@ -1,34 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
 #include <qtest.h>
 
 #include <QSignalSpy>
 
+#include <QtCore/QElapsedTimer>
+
 #include <QtQml/QQmlContext>
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlcomponent.h>
@@ -36,9 +14,10 @@
 #include <QtQuick/qquickview.h>
 #include <private/qquickloader_p.h>
 #include <private/qquickwindowmodule_p.h>
-#include "testhttpserver.h"
-#include "../../shared/util.h"
-#include "../shared/geometrytestutil.h"
+#include <QtQuickTestUtils/private/geometrytestutils_p.h>
+#include <QtQuickTestUtils/private/qmlutils_p.h>
+#include <QtQuickTestUtils/private/testhttpserver_p.h>
+#include <QQmlApplicationEngine>
 
 Q_LOGGING_CATEGORY(lcTests, "qt.quick.tests")
 
@@ -62,11 +41,11 @@ public:
     bool incubated = false;
 
 protected:
-    virtual void timerEvent(QTimerEvent *) {
+    void timerEvent(QTimerEvent *) override {
         incubateFor(15);
     }
 
-    virtual void incubatingObjectCountChanged(int count) {
+    void incubatingObjectCountChanged(int count) override {
         if (count)
             incubated = true;
     }
@@ -87,6 +66,7 @@ private slots:
     void componentToUrl();
     void anchoredLoader();
     void sizeLoaderToItem();
+    void sizeItemToLoader_data();
     void sizeItemToLoader();
     void noResize();
     void networkRequestUrl();
@@ -113,6 +93,7 @@ private slots:
     void asyncToSync1();
     void asyncToSync2();
     void loadedSignal();
+    void selfSetSource();
 
     void parented();
     void sizeBound();
@@ -128,11 +109,24 @@ private slots:
     void rootContext();
     void sourceURLKeepComponent();
 
+    void statusChangeOnlyEmittedOnce();
+
+    void setSourceAndCheckStatus();
+    void loadComponentWithStates();
+    void asyncLoaderRace();
+    void noEngine();
+
+    void stackOverflow();
+    void stackOverflow2();
+    void boundComponent();
+
+    void invalidateContext();
 };
 
 Q_DECLARE_METATYPE(QList<QQmlError>)
 
 tst_QQuickLoader::tst_QQuickLoader()
+    : QQmlDataTest(QT_QMLTEST_DATADIR, FailOnWarningsPolicy::FailOnWarnings)
 {
     qmlRegisterType<SlowComponent>("LoaderTest", 1, 0, "SlowComponent");
     qRegisterMetaType<QList<QQmlError>>();
@@ -178,7 +172,7 @@ void tst_QQuickLoader::sourceOrComponent()
     QCOMPARE(loader->progress(), 1.0);
 
     QCOMPARE(loader->status(), error ? QQuickLoader::Error : QQuickLoader::Ready);
-    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().count(), error ? 0: 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), error ? 0: 1);
 
     if (!error) {
         bool sourceComponentIsChildOfLoader = false;
@@ -212,13 +206,13 @@ void tst_QQuickLoader::sourceOrComponent_data()
     QTest::addColumn<QUrl>("sourceUrl");
     QTest::addColumn<QString>("errorString");
 
-    QTest::newRow("source") << "source" << "source: 'Rect120x60.qml'\n" << testFileUrl("Rect120x60.qml") << "";
-    QTest::newRow("source with subdir") << "source" << "source: 'subdir/Test.qml'\n" << testFileUrl("subdir/Test.qml") << "";
-    QTest::newRow("source with encoded subdir literal") << "source" << "source: 'subdir%2fTest.qml'\n" << testFileUrl("subdir/Test.qml") << "";
-    QTest::newRow("source with encoded subdir optimized binding") << "source" << "source: 'subdir' + '%2fTest.qml'\n" << testFileUrl("subdir/Test.qml") << "";
-    QTest::newRow("source with encoded subdir binding") << "source" << "source: encodeURIComponent('subdir/Test.qml')\n" << testFileUrl("subdir/Test.qml") << "";
+    QTest::newRow("source") << "source" << "source: 'Rect120x60.qml'\n" << QUrl("Rect120x60.qml") << "";
+    QTest::newRow("source with subdir") << "source" << "source: 'subdir/Test.qml'\n" << QUrl("subdir/Test.qml") << "";
+    QTest::newRow("source with encoded subdir literal") << "source" << "source: 'subdir%2fTest.qml'\n" << QUrl("subdir%2FTest.qml") << "";
+    QTest::newRow("source with encoded subdir optimized binding") << "source" << "source: 'subdir' + '%2fTest.qml'\n" << QUrl("subdir%2FTest.qml") << "";
+    QTest::newRow("source with encoded subdir binding") << "source" << "source: encodeURIComponent('subdir/Test.qml')\n" << QUrl("subdir%2FTest.qml") << "";
     QTest::newRow("sourceComponent") << "component" << "Component { id: comp; Rectangle { width: 100; height: 50 } }\n sourceComponent: comp\n" << QUrl() << "";
-    QTest::newRow("invalid source") << "source" << "source: 'IDontExist.qml'\n" << testFileUrl("IDontExist.qml")
+    QTest::newRow("invalid source") << "source" << "source: 'IDontExist.qml'\n" << QUrl("IDontExist.qml")
             << QString(testFileUrl("IDontExist.qml").toString() + ": No such file or directory");
 }
 
@@ -239,12 +233,12 @@ void tst_QQuickLoader::clear()
         QVERIFY(loader != nullptr);
         QVERIFY(loader->item());
         QCOMPARE(loader->progress(), 1.0);
-        QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().count(), 1);
+        QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 1);
 
         QTRY_VERIFY(!loader->item());
         QCOMPARE(loader->progress(), 0.0);
         QCOMPARE(loader->status(), QQuickLoader::Null);
-        QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().count(), 0);
+        QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 0);
     }
     {
         QQmlComponent component(&engine, testFileUrl("/SetSourceComponent.qml"));
@@ -255,14 +249,14 @@ void tst_QQuickLoader::clear()
         QVERIFY(loader);
         QVERIFY(loader->item());
         QCOMPARE(loader->progress(), 1.0);
-        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 
         loader->setSourceComponent(nullptr);
 
         QVERIFY(!loader->item());
         QCOMPARE(loader->progress(), 0.0);
         QCOMPARE(loader->status(), QQuickLoader::Null);
-        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 0);
+        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 0);
     }
     {
         QQmlComponent component(&engine, testFileUrl("/SetSourceComponent.qml"));
@@ -273,14 +267,14 @@ void tst_QQuickLoader::clear()
         QVERIFY(loader);
         QVERIFY(loader->item());
         QCOMPARE(loader->progress(), 1.0);
-        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 
         QMetaObject::invokeMethod(item.data(), "clear");
 
         QVERIFY(!loader->item());
         QCOMPARE(loader->progress(), 0.0);
         QCOMPARE(loader->status(), QQuickLoader::Null);
-        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 0);
+        QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 0);
     }
 }
 
@@ -301,7 +295,7 @@ void tst_QQuickLoader::urlToComponent()
     QTRY_VERIFY(loader != nullptr);
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
-    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 1);
     QCOMPARE(loader->width(), 10.0);
     QCOMPARE(loader->height(), 10.0);
 }
@@ -317,12 +311,12 @@ void tst_QQuickLoader::componentToUrl()
     QVERIFY(loader);
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 
-    loader->setSource(testFileUrl("/Rect120x60.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/Rect120x60.qml"));
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
     QCOMPARE(loader->width(), 120.0);
     QCOMPARE(loader->height(), 60.0);
 }
@@ -389,8 +383,29 @@ void tst_QQuickLoader::sizeLoaderToItem()
     QCOMPARE(rect->height(), 30.0);
 }
 
+void tst_QQuickLoader::sizeItemToLoader_data()
+{
+    QTest::addColumn<QString>("property");
+    QTest::addColumn<int>("value");
+    QTest::addColumn<bool>("atOnce");
+
+    QTest::addRow("none_atonce") << "ignore" << 42 << true;
+    QTest::addRow("width_atonce") << "w" << 42 << true;
+    QTest::addRow("height_atonce") << "h" << 42 << true;
+    QTest::addRow("both_atonce") << "both" << 42 << true;
+
+
+    QTest::addRow("none") << "ignore" << 42 << false;
+    QTest::addRow("width") << "w" << 42 << false;
+    QTest::addRow("height") << "h" << 42 << false;
+    QTest::addRow("both") << "both" << 42 << false;
+}
+
 void tst_QQuickLoader::sizeItemToLoader()
 {
+    QFETCH(QString, property);
+    QFETCH(int, value);
+    QFETCH(bool, atOnce);
     QQmlEngine engine;
     QQmlComponent component(&engine, testFileUrl("/SizeToLoader.qml"));
     QScopedPointer<QQuickLoader> loader(qobject_cast<QQuickLoader*>(component.create()));
@@ -400,17 +415,25 @@ void tst_QQuickLoader::sizeItemToLoader()
 
     QQuickItem *rect = qobject_cast<QQuickItem*>(loader->item());
     QVERIFY(rect);
+    rect->setProperty(property.toUtf8(), value);
     QCOMPARE(rect->width(), 200.0);
     QCOMPARE(rect->height(), 80.0);
 
     // Check resize
     QSizeChangeListener sizeListener(rect);
     const QSizeF size(180, 30);
-    loader->setSize(size);
+    if (atOnce) {
+        loader->setSize(size);
+    } else {
+        loader->setWidth(size.width());
+        loader->setHeight(size.height());
+    }
     QVERIFY2(!sizeListener.isEmpty(), "There should be at least one signal about the size changed");
-    for (const QSizeF sizeOnGeometryChanged : sizeListener) {
-        // Check that we have the correct size on all signals
-        QCOMPARE(sizeOnGeometryChanged, size);
+    if (atOnce) {
+        for (const QSizeF sizeOnGeometryChanged : sizeListener) {
+            // Check that we have the correct size on all signals
+            QCOMPARE(sizeOnGeometryChanged, size);
+        }
     }
     QCOMPARE(rect->width(), size.width());
     QCOMPARE(rect->height(), size.height());
@@ -452,7 +475,7 @@ void tst_QQuickLoader::networkRequestUrl()
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(loader->property("signalCount").toInt(), 1);
-    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 1);
 }
 
 /* XXX Component waits until all dependencies are loaded.  Is this actually possible? */
@@ -483,7 +506,7 @@ void tst_QQuickLoader::networkComponent()
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(loader->status(), QQuickLoader::Ready);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->children().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->children().size(), 1);
 
 }
 
@@ -506,7 +529,7 @@ void tst_QQuickLoader::failNetworkRequest()
     QVERIFY(!loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(loader->property("did_load").toInt(), 123);
-    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().count(), 0);
+    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 0);
 }
 
 void tst_QQuickLoader::active()
@@ -681,6 +704,31 @@ void tst_QQuickLoader::initialPropertyValues_data()
             << QStringList()
             << (QStringList() << "initialValue")
             << (QVariantList() << 6);
+
+    QTest::newRow("ensure required properties are set correctly") << testFileUrl("initialPropertyValues.9.qml")
+            << QStringList()
+            << (QStringList() << "i" << "s")
+            << (QVariantList() << 42 << QLatin1String("hello world"));
+
+    QTest::newRow("required properties only partially set =") << testFileUrl("initialPropertyValues.10.qml")
+            << (QStringList() << QString(testFileUrl("RequiredPropertyValuesComponent.qml").toString() +  QLatin1String(":6:5: Required property s was not initialized")))
+            << (QStringList() << "i" << "s")
+            << (QVariantList() << 0 << QLatin1String(""));
+
+    QTest::newRow("source url changed, previously initial properties are discared") << testFileUrl("initialPropertyValues.11.qml")
+                                                                                    << QStringList()
+                                                                                    << (QStringList() << "oldi" << "i")
+                                                                                    << (QVariantList() << 12 << 42);
+
+    QTest::newRow("ensure initial properties aren't disposed after active = true") << testFileUrl("initialPropertyValues.12.qml")
+            << QStringList()
+            << (QStringList() << "i")
+            << (QVariantList() << 12);
+
+    QTest::newRow("initial property errors get reported") << testFileUrl("initialPropertyTriggerException.qml")
+                                                          << (QStringList() << "^.*:10: Error: Cannot assign JavaScript function to int")
+                                                          << QStringList()
+                                                          << QVariantList();
 }
 
 void tst_QQuickLoader::initialPropertyValues()
@@ -692,8 +740,8 @@ void tst_QQuickLoader::initialPropertyValues()
 
     ThreadedTestHTTPServer server(dataDirectory());
 
-    foreach (const QString &warning, expectedWarnings)
-        QTest::ignoreMessage(QtWarningMsg, warning.toLatin1().constData());
+    for (const QString &warning : std::as_const(expectedWarnings))
+        QTest::ignoreMessage(QtWarningMsg, QRegularExpression(warning.toLatin1().constData()));
 
     QQmlEngine engine;
     QQmlComponent component(&engine, qmlFile);
@@ -738,8 +786,10 @@ void tst_QQuickLoader::initialPropertyValuesError_data()
     QTest::newRow("nonexistent source url") << testFileUrl("initialPropertyValues.error.2.qml")
             << (QStringList() << QString(testFileUrl("NonexistentSourceComponent.qml").toString() + ": No such file or directory"));
 
-    QTest::newRow("invalid source url") << testFileUrl("initialPropertyValues.error.3.qml")
-            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString() + ":5:1: Expected token `:'"));
+    QTest::newRow("invalid source url")
+            << testFileUrl("initialPropertyValues.error.3.qml")
+            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString()
+                                         + ":4:5: Incomplete binding, expected token `:` or `{`"));
 
     QTest::newRow("invalid initial property values object with invalid property access") << testFileUrl("initialPropertyValues.error.4.qml")
             << (QStringList() << QString(testFileUrl("initialPropertyValues.error.4.qml").toString() + ":7:5: QML Loader: setSource: value is not an object")
@@ -751,7 +801,7 @@ void tst_QQuickLoader::initialPropertyValuesError()
     QFETCH(QUrl, qmlFile);
     QFETCH(QStringList, expectedWarnings);
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : std::as_const(expectedWarnings))
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QQmlEngine engine;
@@ -781,8 +831,8 @@ void tst_QQuickLoader::deleteComponentCrash()
     QCOMPARE(loader->status(), QQuickLoader::Ready);
     QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
     QCoreApplication::processEvents();
-    QTRY_COMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
-    QCOMPARE(loader->source(), testFileUrl("BlueRect.qml"));
+    QTRY_COMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
+    QCOMPARE(loader->source(), QUrl("BlueRect.qml"));
 }
 
 void tst_QQuickLoader::nonItem()
@@ -858,8 +908,8 @@ void tst_QQuickLoader::implicitSize()
     QCOMPARE(loader->property("implicitWidth").toReal(), 200.);
     QCOMPARE(loader->property("implicitHeight").toReal(), 300.);
 
-    QCOMPARE(implWidthSpy.count(), 1);
-    QCOMPARE(implHeightSpy.count(), 1);
+    QCOMPARE(implWidthSpy.size(), 1);
+    QCOMPARE(implHeightSpy.size(), 1);
 }
 
 void tst_QQuickLoader::QTBUG_17114()
@@ -884,8 +934,10 @@ void tst_QQuickLoader::asynchronous_data()
     QTest::newRow("Non-existent component") << testFileUrl("IDoNotExist.qml")
             << (QStringList() << QString(testFileUrl("IDoNotExist.qml").toString() + ": No such file or directory"));
 
-    QTest::newRow("Invalid component") << testFileUrl("InvalidSourceComponent.qml")
-            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString() + ":5:1: Expected token `:'"));
+    QTest::newRow("Invalid component")
+            << testFileUrl("InvalidSourceComponent.qml")
+            << (QStringList() << QString(testFileUrl("InvalidSourceComponent.qml").toString()
+                                         + ":4:5: Incomplete binding, expected token `:` or `{`"));
 }
 
 void tst_QQuickLoader::asynchronous()
@@ -894,9 +946,9 @@ void tst_QQuickLoader::asynchronous()
     QFETCH(QStringList, expectedWarnings);
 
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -906,7 +958,7 @@ void tst_QQuickLoader::asynchronous()
     QQuickLoader *loader = root->findChild<QQuickLoader*>("loader");
     QVERIFY(loader);
 
-    foreach (const QString &warning, expectedWarnings)
+    for (const QString &warning : std::as_const(expectedWarnings))
         QTest::ignoreMessage(QtWarningMsg, warning.toUtf8().constData());
 
     QVERIFY(!loader->item());
@@ -933,9 +985,9 @@ void tst_QQuickLoader::asynchronous()
 void tst_QQuickLoader::asynchronous_clear()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -962,7 +1014,7 @@ void tst_QQuickLoader::asynchronous_clear()
 
     QCOMPARE(loader->progress(), 0.0);
     QCOMPARE(loader->status(), QQuickLoader::Null);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 0);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 0);
 
     // check loading component
     root->setProperty("comp", "BigComponent.qml");
@@ -975,15 +1027,15 @@ void tst_QQuickLoader::asynchronous_clear()
     QTRY_VERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(loader->status(), QQuickLoader::Ready);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 }
 
 void tst_QQuickLoader::simultaneousSyncAsync()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("simultaneous.qml"));
@@ -1013,9 +1065,9 @@ void tst_QQuickLoader::simultaneousSyncAsync()
 void tst_QQuickLoader::asyncToSync1()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -1039,15 +1091,15 @@ void tst_QQuickLoader::asyncToSync1()
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(loader->status(), QQuickLoader::Ready);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 }
 
 void tst_QQuickLoader::asyncToSync2()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     QQmlComponent component(&engine, testFileUrl("asynchronous.qml"));
@@ -1071,15 +1123,15 @@ void tst_QQuickLoader::asyncToSync2()
     QVERIFY(loader->item());
     QCOMPARE(loader->progress(), 1.0);
     QCOMPARE(loader->status(), QQuickLoader::Ready);
-    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().count(), 1);
+    QCOMPARE(static_cast<QQuickItem*>(loader)->childItems().size(), 1);
 }
 
 void tst_QQuickLoader::loadedSignal()
 {
     QQmlEngine engine;
-    PeriodicIncubationController *controller = new PeriodicIncubationController;
+    QScopedPointer<PeriodicIncubationController> controller(new PeriodicIncubationController);
     QQmlIncubationController *previous = engine.incubationController();
-    engine.setIncubationController(controller);
+    engine.setIncubationController(controller.data());
     delete previous;
 
     {
@@ -1118,6 +1170,17 @@ void tst_QQuickLoader::loadedSignal()
     }
 }
 
+void tst_QQuickLoader::selfSetSource()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("selfSetSourceTest.qml"));
+
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(obj);
+
+    QTRY_COMPARE(obj->property("source").toUrl(), testFileUrl("empty.qml"));
+}
+
 void tst_QQuickLoader::parented()
 {
     QQmlEngine engine;
@@ -1268,7 +1331,7 @@ void tst_QQuickLoader::sourceComponentGarbageCollection()
     if (spy.isEmpty())
         QVERIFY(spy.wait());
 
-    QCOMPARE(spy.count(), 1);
+    QCOMPARE(spy.size(), 1);
 }
 
 // QTBUG-51995
@@ -1400,7 +1463,7 @@ void tst_QQuickLoader::sourceURLKeepComponent()
                       dataDirectoryUrl());
 
     QScopedPointer<QQuickLoader> loader(qobject_cast<QQuickLoader*>(component.create()));
-    loader->setSource(testFileUrl("/Rect120x60.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/Rect120x60.qml"));
 
     QVERIFY(loader);
     QVERIFY(loader->item());
@@ -1426,7 +1489,7 @@ void tst_QQuickLoader::sourceURLKeepComponent()
     QCOMPARE(sourceComponent.data(), loader->sourceComponent());
 
     //Ensure changing source url causes component to be recreated when inactive
-    loader->setSource(testFileUrl("/BlueRect.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/BlueRect.qml"));
 
     loader->setActive(true);
     QVERIFY(loader->item());
@@ -1436,11 +1499,183 @@ void tst_QQuickLoader::sourceURLKeepComponent()
     QVERIFY(sourceComponent.data() != newSourceComponent.data());
 
     //Ensure changing source url causes component to be recreated when active
-    loader->setSource(testFileUrl("/Rect120x60.qml"));
+    loader->setSourceWithoutResolve(testFileUrl("/Rect120x60.qml"));
     QVERIFY(loader->sourceComponent() != newSourceComponent.data());
 
 }
 
+// QTBUG-82002
+void tst_QQuickLoader::statusChangeOnlyEmittedOnce()
+{
+    QQmlApplicationEngine engine;
+    auto url = testFileUrl("statusChanged.qml");
+    engine.load(url);
+    auto root = engine.rootObjects().at(0);
+    QVERIFY(root);
+    QTRY_COMPARE(QQuickLoader::Status(root->property("status").toInt()), QQuickLoader::Ready);
+    QCOMPARE(root->property("statusChangedCounter").toInt(), 2); // 1xLoading + 1xReady*/
+}
+
+void tst_QQuickLoader::setSourceAndCheckStatus()
+{
+    QQmlApplicationEngine engine;
+    auto url = testFileUrl("setSourceAndCheckStatus.qml");
+    engine.load(url);
+    auto root = engine.rootObjects().at(0);
+    QVERIFY(root);
+
+    QQuickLoader *loader = root->findChild<QQuickLoader *>();
+    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant::fromValue(QStringLiteral("./RedRect.qml"))));
+    QCOMPARE(loader->status(), QQuickLoader::Ready);
+
+    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant::fromValue(QStringLiteral(""))));
+    QCOMPARE(loader->status(), QQuickLoader::Null);
+
+    QMetaObject::invokeMethod(loader, "load", Q_ARG(QVariant, QVariant(QUrl())));
+    QCOMPARE(loader->status(), QQuickLoader::Null);
+}
+
+void tst_QQuickLoader::loadComponentWithStates()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData(QByteArray("import QtQuick\n"
+                                 "Loader {\n"
+                                     "id: loader\n"
+                                     "property int createdObjCount: 0\n"
+                                     "states: [ State { when: true; PropertyChanges { target: loader; sourceComponent: myComp } } ]\n"
+                                     "Component { id: myComp; Item { Component.onCompleted: { ++createdObjCount } } }\n"
+                                 "}" )
+            , dataDirectoryUrl());
+    QScopedPointer<QQuickLoader> loader(qobject_cast<QQuickLoader*>(component.create()));
+    QTest::qWait(200);
+    QTRY_VERIFY(loader != nullptr);
+    QVERIFY(loader->item());
+    QCOMPARE(static_cast<QQuickItem*>(loader.data())->childItems().size(), 1);
+    QCOMPARE(loader->property("createdObjCount").toInt(), 1);
+}
+
+void tst_QQuickLoader::asyncLoaderRace()
+{
+    QQmlApplicationEngine engine;
+    auto url = testFileUrl("loader-async-race.qml");
+    engine.load(url);
+    auto root = engine.rootObjects().at(0);
+    QVERIFY(root);
+
+    QQuickLoader *loader = root->findChild<QQuickLoader *>();
+    QCOMPARE(loader->active(), false);
+    QCOMPARE(loader->status(), QQuickLoader::Null);
+    QCOMPARE(loader->item(), nullptr);
+
+    QSignalSpy spy(loader, &QQuickLoader::itemChanged);
+    QVERIFY(!spy.wait(100));
+    QCOMPARE(loader->item(), nullptr);
+}
+
+void tst_QQuickLoader::noEngine()
+{
+    QQmlEngine engine;
+    const QUrl url = testFileUrl("noEngine.qml");
+    QQmlComponent component(&engine, url);
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> o(component.create());
+
+    const QString message = url.toString()
+            + QStringLiteral(":27:13: QML Loader: createComponent: Cannot find a QML engine.");
+    QTest::ignoreMessage(QtWarningMsg, qPrintable(message));
+    QTRY_COMPARE(o->property("changes").toInt(), 1);
+}
+
+static void qTestForOverflow(const QUrl &url)
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, url);
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    const QString message = url.toString() + QStringLiteral(": Maximum call stack size exceeded.");
+    QTest::ignoreMessage(QtCriticalMsg, qPrintable(message));
+    QScopedPointer<QObject> o(component.create());
+}
+
+void tst_QQuickLoader::stackOverflow()
+{
+    auto t = QThread::create(qTestForOverflow, testFileUrl("overflow.qml"));
+    t->setStackSize(1024 * 1024);
+    t->start();
+    t->wait();
+}
+
+void tst_QQuickLoader::stackOverflow2()
+{
+    auto t = QThread::create(qTestForOverflow, testFileUrl("overflow2.qml"));
+    t->setStackSize(1024 * 1024);
+    t->start();
+    t->wait();
+}
+
+void tst_QQuickLoader::boundComponent()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("boundComponent.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> o(component.create());
+    QCOMPARE(o->objectName(), QStringLiteral("loaded"));
+}
+
+class CppModel : public QAbstractListModel
+{
+public:
+    virtual int rowCount(const QModelIndex &) const override {return 1;}
+    virtual QVariant data(const QModelIndex &, int) const override { return "test"; }
+
+    void reset () {beginResetModel(); endResetModel();}
+};
+
+class RootData : public QObject
+{
+    Q_OBJECT
+public:
+    RootData() : object(std::make_unique<QObject>())
+    {
+        object->setObjectName("objectName");
+    }
+
+    Q_INVOKABLE QString getValue() const {  return object->objectName(); }
+
+    void deleteObject() { object.reset(); }
+
+private:
+    std::unique_ptr<QObject> object;
+};
+
+void tst_QQuickLoader::invalidateContext()
+{
+    CppModel model;
+    RootData rootData;
+
+    QQmlEngine engine;
+    QQmlContext *rootContext = engine.rootContext();
+    rootContext->setContextProperty("cppModel", &model);
+    rootContext->setContextProperty("loaderActive", true);
+    rootContext->setContextProperty("rootData", &rootData);
+
+    QQmlComponent component(&engine, testFileUrl("invalidateContext.qml"));\
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    QTest::ignoreMessage(QtDebugMsg, "Repeater constructed");
+    QTest::ignoreMessage(QtDebugMsg, "updating text");
+    QScopedPointer<QObject> o(component.create());
+
+    QTest::ignoreMessage(QtDebugMsg, "Repeater destroyed");
+    rootContext->setContextProperty("loaderActive", false);
+
+    // The object should never be used anymore, as loader is inactive
+    rootData.deleteObject();
+
+    // Should not trigger any re-evaluation (which would crash in getValue())
+    model.reset();
+}
+
 QTEST_MAIN(tst_QQuickLoader)
 
 #include "tst_qquickloader.moc"
