@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/qtest.h>
 #include <QtTest/qsignalspy.h>
@@ -104,13 +68,18 @@ private slots:
 
 private:
     QTemporaryDir tempDir;
+    QString tempDirCanonicalPath;
     QScopedPointer<QFile> tempFile1;
+    QString tempFile1CanonicalPath;
     QScopedPointer<QFile> tempFile2;
+    QString tempFile2CanonicalPath;
     QDir tempSubDir1;
+    QString tempSubDir1CanonicalPath;
     QDir tempSubSubDir;
     QScopedPointer<QFile> tempSubFile1;
     QScopedPointer<QFile> tempSubFile2;
     QDir tempSubDir2;
+    QString tempSubDir2CanonicalPath;
     QDir oldCurrentDir;
 };
 
@@ -124,11 +93,12 @@ void tst_QQuickFolderDialogImpl::initTestCase()
     QQmlDataTest::initTestCase();
 
     QVERIFY(tempDir.isValid());
+    tempDirCanonicalPath = QFileInfo(tempDir.path()).canonicalFilePath();
     // QTEST_QUICKCONTROLS_MAIN constructs the test case object once,
     // and then calls qRun() for each style, and qRun() calls initTestCase().
     // So, we need to check if we've already made the temporary directory.
     // Note that this is only necessary if the test is run with more than one style.
-    if (!QDir(tempDir.path()).isEmpty())
+    if (!QDir(tempDirCanonicalPath).isEmpty())
         return;
 
     /*
@@ -143,39 +113,43 @@ void tst_QQuickFolderDialogImpl::initTestCase()
             ├── file1.txt
             └── file2.txt
     */
-    tempSubDir1 = QDir(tempDir.path());
+    tempSubDir1 = QDir(tempDirCanonicalPath);
     QVERIFY2(tempSubDir1.mkdir("sub-dir-1"), qPrintable(QString::fromLatin1(
         "Failed to make sub-directory \"sub-dir-1\" in %1. Permissions are: %2")
-            .arg(tempDir.path(), QDebug::toString(QFileInfo(tempDir.path()).permissions()))));
+            .arg(tempDirCanonicalPath, QDebug::toString(QFileInfo(tempDirCanonicalPath).permissions()))));
     QVERIFY(tempSubDir1.cd("sub-dir-1"));
+    tempSubDir1CanonicalPath = tempSubDir1.canonicalPath();
+    QVERIFY(!tempSubDir1CanonicalPath.isEmpty());
 
-    tempSubSubDir = QDir(tempSubDir1.path());
+    tempSubSubDir = QDir(tempSubDir1CanonicalPath);
     QVERIFY2(tempSubSubDir.mkdir("sub-sub-dir"), qPrintable(QString::fromLatin1(
         "Failed to make sub-directory \"sub-sub-dir\" in %1. Permissions are: %2")
-            .arg(tempSubDir1.path(), QDebug::toString(QFileInfo(tempSubDir1.path()).permissions()))));
+            .arg(tempSubDir1CanonicalPath, QDebug::toString(QFileInfo(tempSubDir1CanonicalPath).permissions()))));
     QVERIFY(tempSubSubDir.cd("sub-sub-dir"));
 
-    tempSubFile1.reset(new QFile(tempSubDir1.path() + "/sub-file1.txt"));
+    tempSubFile1.reset(new QFile(tempSubDir1CanonicalPath + "/sub-file1.txt"));
     QVERIFY(tempSubFile1->open(QIODevice::ReadWrite));
 
-    tempSubFile2.reset(new QFile(tempSubDir1.path() + "/sub-file2.txt"));
+    tempSubFile2.reset(new QFile(tempSubDir1CanonicalPath + "/sub-file2.txt"));
     QVERIFY(tempSubFile2->open(QIODevice::ReadWrite));
 
-    tempSubDir2 = QDir(tempDir.path());
+    tempSubDir2 = QDir(tempDirCanonicalPath);
     QVERIFY2(tempSubDir2.mkdir("sub-dir-2"), qPrintable(QString::fromLatin1(
         "Failed to make sub-directory \"sub-dir-2\" in %1. Permissions are: %2")
-            .arg(tempDir.path(), QDebug::toString(QFileInfo(tempDir.path()).permissions()))));
+            .arg(tempDirCanonicalPath, QDebug::toString(QFileInfo(tempDirCanonicalPath).permissions()))));
     QVERIFY(tempSubDir2.cd("sub-dir-2"));
+    tempSubDir2CanonicalPath = tempSubDir2.canonicalPath();
+    QVERIFY(!tempSubDir2CanonicalPath.isEmpty());
 
-    tempFile1.reset(new QFile(tempDir.path() + "/file1.txt"));
+    tempFile1.reset(new QFile(tempDirCanonicalPath + "/file1.txt"));
     QVERIFY(tempFile1->open(QIODevice::ReadWrite));
 
-    tempFile2.reset(new QFile(tempDir.path() + "/file2.txt"));
+    tempFile2.reset(new QFile(tempDirCanonicalPath + "/file2.txt"));
     QVERIFY(tempFile2->open(QIODevice::ReadWrite));
 
     // Ensure that each test starts off in the temporary directory.
     oldCurrentDir = QDir::current();
-    QDir::setCurrent(tempDir.path());
+    QDir::setCurrent(tempDirCanonicalPath);
 }
 
 void tst_QQuickFolderDialogImpl::cleanupTestCase()
@@ -186,6 +160,7 @@ void tst_QQuickFolderDialogImpl::cleanupTestCase()
 
 void tst_QQuickFolderDialogImpl::defaults()
 {
+    QTest::failOnWarning(QRegularExpression(".*"));
     QQuickApplicationHelper helper(this, "folderDialog.qml");
     QVERIFY2(helper.ready, helper.failureMessage());
 
@@ -207,9 +182,9 @@ void tst_QQuickFolderDialogImpl::defaults()
     QVERIFY(quickDialog);
     QTRY_VERIFY(quickDialog->isOpened());
     QVERIFY(quickDialog);
-    COMPARE_URL(quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1.path()));
+    COMPARE_URL(quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
     COMPARE_URL(quickDialog->currentFolder(), QUrl::fromLocalFile(QDir().absolutePath()));
-    COMPARE_URL(dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1.path()));
+    COMPARE_URL(dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
     COMPARE_URL(dialog->currentFolder(), QUrl::fromLocalFile(QDir().absolutePath()));
     QCOMPARE(quickDialog->title(), QString());
 }
@@ -221,9 +196,9 @@ class FolderDialogSignalHelper
 public:
     FolderDialogSignalHelper(const FolderDialogTestHelper &dialogTestHelper) :
         dialogSelectedFolderChangedSpy(dialogTestHelper.dialog, SIGNAL(selectedFolderChanged())),
-        quickDialogSelectedFolderChangedSpy(dialogTestHelper.quickDialog, SIGNAL(selectedFolderChanged(const QUrl &))),
+        quickDialogSelectedFolderChangedSpy(dialogTestHelper.quickDialog, SIGNAL(selectedFolderChanged(QUrl))),
         dialogCurrentFolderChangedSpy(dialogTestHelper.dialog, SIGNAL(currentFolderChanged())),
-        quickDialogCurrentFolderChangedSpy(dialogTestHelper.quickDialog, SIGNAL(currentFolderChanged(const QUrl &)))
+        quickDialogCurrentFolderChangedSpy(dialogTestHelper.quickDialog, SIGNAL(currentFolderChanged(QUrl)))
     {
         if (!dialogSelectedFolderChangedSpy.isValid())
             errorMessage = "selectedFolderChanged signal of dialog is not valid";
@@ -259,18 +234,18 @@ void tst_QQuickFolderDialogImpl::chooseFolderViaStandardButtons()
     QVERIFY(folderDialogListView);
     QQuickFileDialogDelegate *delegate = nullptr;
     QTRY_VERIFY(findViewDelegateItem(folderDialogListView, 1, delegate));
-    COMPARE_URL(delegate->file(), QUrl::fromLocalFile(tempSubDir2.path()));
+    COMPARE_URL(delegate->file(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
     QVERIFY(clickButton(delegate));
     // currentFolder shouldn't change just from a single click.
-    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
-    COMPARE_URL(dialogHelper.quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
-    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
+    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
+    COMPARE_URL(dialogHelper.quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
+    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
     // Only selectedFile-related signals should be emitted.
-    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.count(), 0);
-    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.count(), 0);
+    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.size(), 0);
+    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.size(), 0);
 
     // Click the "Open" button.
     QVERIFY(dialogHelper.quickDialog->footer());
@@ -279,12 +254,12 @@ void tst_QQuickFolderDialogImpl::chooseFolderViaStandardButtons()
     QQuickAbstractButton* openButton = findDialogButton(dialogButtonBox, "Open");
     QVERIFY(openButton);
     QVERIFY(clickButton(openButton));
-    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
-    COMPARE_URL(dialogHelper.quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
-    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.count(), 0);
-    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.count(), 0);
+    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
+    COMPARE_URL(dialogHelper.quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
+    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.size(), 0);
+    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.size(), 0);
     QTRY_VERIFY(!dialogHelper.quickDialog->isVisible());
     QVERIFY(!dialogHelper.dialog->isVisible());
 }
@@ -295,11 +270,11 @@ void tst_QQuickFolderDialogImpl::bindCurrentFolder_data()
     QTest::addColumn<QUrl>("expectedFolder");
     QTest::addColumn<QStringList>("expectedVisibleFiles");
 
-    const auto currentDirUrl = QUrl::fromLocalFile(QDir::current().path());
-    const auto tempSubDirUrl = QUrl::fromLocalFile(tempSubDir1.path());
+    const auto currentDirUrl = QUrl::fromLocalFile(QDir::current().canonicalPath());
+    const auto tempSubDirUrl = QUrl::fromLocalFile(tempSubDir1CanonicalPath);
     const auto tempSubFile1Url = QUrl::fromLocalFile(tempSubFile1->fileName());
 
-    const QStringList currentDirFiles = { tempSubDir1.path(), tempSubDir2.path() };
+    const QStringList currentDirFiles = { tempSubDir1CanonicalPath, tempSubDir2CanonicalPath };
     const QStringList tempSubDirFiles = { tempSubSubDir.path() };
 
     // Setting the folder to "sub-dir-1" should result in "sub-sub-dir" being visible.
@@ -353,16 +328,16 @@ void tst_QQuickFolderDialogImpl::changeFolderViaDoubleClick()
     QVERIFY(folderDialogListView);
     QQuickFileDialogDelegate *delegate = nullptr;
     QTRY_VERIFY(findViewDelegateItem(folderDialogListView, 1, delegate));
-    COMPARE_URL(delegate->file(), QUrl::fromLocalFile(tempSubDir2.path()))
+    COMPARE_URL(delegate->file(), QUrl::fromLocalFile(tempSubDir2CanonicalPath))
     QVERIFY(doubleClickButton(delegate));
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2.path()))
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath))
     COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl());
     // selectedFolder is set to the folder when clicked and then set to an empty URL after
     // the double click.
-    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.count(), 2);
-    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.count(), 2);
-    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.count(), 1);
+    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.size(), 2);
+    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.size(), 2);
+    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.size(), 1);
     // Since we only changed the current folder, the dialog should still be open.
     QVERIFY(dialogHelper.dialog->isVisible());
 
@@ -379,10 +354,12 @@ void tst_QQuickFolderDialogImpl::changeFolderViaTextEdit()
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
+#if QT_CONFIG(shortcut)
     // Get the text edit visible with Ctrl+L.
     const auto editPathKeySequence = QKeySequence(Qt::CTRL | Qt::Key_L);
-    QTest::keySequence(dialogHelper.window(), editPathKeySequence);
+    QTest::keySequence(dialogHelper.popupWindow(), editPathKeySequence);
     auto breadcrumbBar = dialogHelper.quickDialog->findChild<QQuickFolderBreadcrumbBar*>();
     QVERIFY(breadcrumbBar);
     QVERIFY(breadcrumbBar->textField()->isVisible());
@@ -390,13 +367,14 @@ void tst_QQuickFolderDialogImpl::changeFolderViaTextEdit()
     QCOMPARE(breadcrumbBar->textField()->selectedText(), breadcrumbBar->textField()->text());
 
     // Enter the path to the folder in the text edit.
-    enterText(dialogHelper.window(), tempSubDir2.path());
-    QCOMPARE(breadcrumbBar->textField()->text(), tempSubDir2.path());
+    enterText(dialogHelper.popupWindow(), tempSubDir2CanonicalPath);
+    QCOMPARE(breadcrumbBar->textField()->text(), tempSubDir2CanonicalPath);
+#endif
 
     // Hit enter to accept.
-    QTest::keyClick(dialogHelper.window(), Qt::Key_Return);
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
-    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
+    QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Return);
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
+    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
     // We changed into a directory with no folders, so the selected folder should be invalid.
     COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl());
     COMPARE_URL(dialogHelper.quickDialog->selectedFolder(), QUrl());
@@ -416,26 +394,27 @@ void tst_QQuickFolderDialogImpl::changeFolderViaEnter()
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     // The first delegate in the view should be selected and have focus.
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
     auto folderDialogListView = dialogHelper.quickDialog->findChild<QQuickListView*>("folderDialogListView");
     QVERIFY(folderDialogListView);
     QQuickFileDialogDelegate *subDir1Delegate = nullptr;
     QTRY_VERIFY(findViewDelegateItem(folderDialogListView, 0, subDir1Delegate));
-    COMPARE_URL(subDir1Delegate->file(), QUrl::fromLocalFile(tempSubDir1.path()));
+    COMPARE_URL(subDir1Delegate->file(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
     QVERIFY(subDir1Delegate->hasActiveFocus());
 
     // Select the delegate by pressing enter.
     const FolderDialogSignalHelper signalHelper(dialogHelper);
     QVERIFY2(signalHelper.errorMessage.isEmpty(), signalHelper.errorMessage);
-    QTest::keyClick(dialogHelper.window(), Qt::Key_Return);
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir1.path()));
+    QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Return);
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
     COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubSubDir.path()));
-    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.count(), 1);
-    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.count(), 1);
+    QCOMPARE(signalHelper.dialogSelectedFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.quickDialogSelectedFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.dialogCurrentFolderChangedSpy.size(), 1);
+    QCOMPARE(signalHelper.quickDialogCurrentFolderChangedSpy.size(), 1);
     // Since we only changed the current folder, the dialog should still be open.
     QVERIFY(dialogHelper.dialog->isVisible());
 
@@ -452,10 +431,13 @@ void tst_QQuickFolderDialogImpl::cancelDialogWhileTextEditHasFocus()
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
+#if QT_CONFIG(shortcut)
     // Get the text edit visible with Ctrl+L.
     const auto editPathKeySequence = QKeySequence(Qt::CTRL | Qt::Key_L);
-    QTest::keySequence(dialogHelper.window(), editPathKeySequence);
+    QTest::keySequence(dialogHelper.popupWindow(), editPathKeySequence);
+#endif
     auto breadcrumbBar = dialogHelper.quickDialog->findChild<QQuickFolderBreadcrumbBar*>();
     QVERIFY(breadcrumbBar);
     QVERIFY(breadcrumbBar->textField()->isVisible());
@@ -484,7 +466,7 @@ void tst_QQuickFolderDialogImpl::goUp()
 {
     // Open the dialog, starting off in "sub-dir-1".
     FolderDialogTestHelper dialogHelper(this, "bindCurrentFolder.qml", {},
-        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir1.path()) }});
+        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir1CanonicalPath) }});
     QVERIFY2(dialogHelper.isWindowInitialized(), dialogHelper.failureMessage());
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
@@ -496,35 +478,38 @@ void tst_QQuickFolderDialogImpl::goUp()
     auto barListView = qobject_cast<QQuickListView*>(breadcrumbBar->contentItem());
     QVERIFY(barListView);
     if (QQuickTest::qIsPolishScheduled(barListView))
-        QVERIFY(QQuickTest::qWaitForItemPolished(barListView));
+        QVERIFY(QQuickTest::qWaitForPolish(barListView));
     QVERIFY(clickButton(breadcrumbBar->upButton()));
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
     // The previous directory that we were in should now be selected (matches e.g. Windows and Ubuntu).
-    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1.path()));
+    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
     auto folderDialogListView = dialogHelper.quickDialog->findChild<QQuickListView*>("folderDialogListView");
     QVERIFY(folderDialogListView);
     QQuickFileDialogDelegate *subDirDelegate = nullptr;
     QTRY_VERIFY(findViewDelegateItem(folderDialogListView, 0, subDirDelegate));
     QCOMPARE(subDirDelegate->isHighlighted(), true);
 
+#if QT_CONFIG(shortcut)
     // Go up a directory via the keyboard shortcut.
     const auto goUpKeySequence = QKeySequence(Qt::ALT | Qt::Key_Up);
     QTest::keySequence(dialogHelper.window(), goUpKeySequence);
-    QDir tempParentDir(tempDir.path());
+    QDir tempParentDir(tempDirCanonicalPath);
     QVERIFY(tempParentDir.cdUp());
     COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempParentDir.path()));
-    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempDir.path()));
+    COMPARE_URL(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
+#endif
 }
 
 void tst_QQuickFolderDialogImpl::goUpWhileTextEditHasFocus()
 {
     // Open the dialog, starting off in "sub-dir-1".
     FolderDialogTestHelper dialogHelper(this, "bindCurrentFolder.qml", {},
-        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir1.path()) }});
+        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir1CanonicalPath) }});
     QVERIFY2(dialogHelper.isWindowInitialized(), dialogHelper.failureMessage());
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     // Go up a directory via the button next to the breadcrumb bar.
     auto breadcrumbBar = dialogHelper.quickDialog->findChild<QQuickFolderBreadcrumbBar*>();
@@ -532,20 +517,20 @@ void tst_QQuickFolderDialogImpl::goUpWhileTextEditHasFocus()
     auto barListView = qobject_cast<QQuickListView*>(breadcrumbBar->contentItem());
     QVERIFY(barListView);
     if (QQuickTest::qIsPolishScheduled(barListView))
-        QVERIFY(QQuickTest::qWaitForItemPolished(barListView));
+        QVERIFY(QQuickTest::qWaitForPolish(barListView));
     QVERIFY(clickButton(breadcrumbBar->upButton()));
     // The path should have changed to the parent directory.
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
     // The text edit should be hidden when it loses focus.
     QVERIFY(!breadcrumbBar->textField()->hasActiveFocus());
     QVERIFY(!breadcrumbBar->textField()->isVisible());
     // The focus should be given to the first delegate.
-    QVERIFY(dialogHelper.window()->activeFocusItem());
+    QVERIFY(dialogHelper.popupWindow()->activeFocusItem());
     auto folderDialogListView = dialogHelper.quickDialog->findChild<QQuickListView*>("folderDialogListView");
     QVERIFY(folderDialogListView);
     QQuickFileDialogDelegate *firstDelegate = nullptr;
     QTRY_VERIFY(findViewDelegateItem(folderDialogListView, 0, firstDelegate));
-    QCOMPARE(dialogHelper.window()->activeFocusItem(), firstDelegate);
+    QCOMPARE(dialogHelper.popupWindow()->activeFocusItem(), firstDelegate);
 }
 
 void tst_QQuickFolderDialogImpl::goIntoLargeFolder()
@@ -582,7 +567,7 @@ void tst_QQuickFolderDialogImpl::goIntoLargeFolder()
     auto folderDialogListView = dialogHelper.quickDialog->findChild<QQuickListView*>("folderDialogListView");
     QVERIFY(folderDialogListView);
     if (QQuickTest::qIsPolishScheduled(folderDialogListView))
-        QVERIFY(QQuickTest::qWaitForItemPolished(folderDialogListView));
+        QVERIFY(QQuickTest::qWaitForPolish(folderDialogListView));
     // Just to be safe, make sure it's at least twice as big.
     if (folderDialogListView->contentItem()->height() < folderDialogListView->height() * 2) {
         QSKIP(qPrintable(QString::fromLatin1("Expected height of folderDialogListView's contentItem (%1)" \
@@ -610,7 +595,7 @@ void tst_QQuickFolderDialogImpl::clickOnBreadcrumb()
 {
     // Open the dialog, starting off in "sub-dir-1".
     FolderDialogTestHelper dialogHelper(this, "bindCurrentFolder.qml", {},
-        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir1.path()) }});
+        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir1CanonicalPath) }});
     QVERIFY2(dialogHelper.isWindowInitialized(), dialogHelper.failureMessage());
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
@@ -631,10 +616,10 @@ void tst_QQuickFolderDialogImpl::clickOnBreadcrumb()
     QTRY_VERIFY(findViewDelegateItem(breadcrumbBarListView, breadcrumbBarListView->count() - 3, breadcrumbDelegate));
 
     QVERIFY(clickButton(breadcrumbDelegate));
-    QCOMPARE(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
-    QCOMPARE(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
-    QCOMPARE(dialogHelper.quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1.path()));
-    QCOMPARE(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1.path()));
+    QCOMPARE(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
+    QCOMPARE(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
+    QCOMPARE(dialogHelper.quickDialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
+    QCOMPARE(dialogHelper.dialog->selectedFolder(), QUrl::fromLocalFile(tempSubDir1CanonicalPath));
 }
 
 void tst_QQuickFolderDialogImpl::keyAndShortcutHandling()
@@ -645,36 +630,43 @@ void tst_QQuickFolderDialogImpl::keyAndShortcutHandling()
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
+#if QT_CONFIG(shortcut)
     // Get the text edit visible with Ctrl+L.
     const auto editPathKeySequence = QKeySequence(Qt::CTRL | Qt::Key_L);
-    QTest::keySequence(dialogHelper.window(), editPathKeySequence);
+    QTest::keySequence(dialogHelper.popupWindow(), editPathKeySequence);
+#endif
     auto breadcrumbBar = dialogHelper.quickDialog->findChild<QQuickFolderBreadcrumbBar*>();
     QVERIFY(breadcrumbBar);
     QVERIFY(breadcrumbBar->textField()->isVisible());
     QCOMPARE(breadcrumbBar->textField()->text(), dialogHelper.dialog->currentFolder().toLocalFile());
     QCOMPARE(breadcrumbBar->textField()->selectedText(), breadcrumbBar->textField()->text());
 
+#if QT_CONFIG(shortcut)
     // Ctrl+L shouldn't hide it.
-    QTest::keySequence(dialogHelper.window(), editPathKeySequence);
+    QTest::keySequence(dialogHelper.popupWindow(), editPathKeySequence);
     QVERIFY(breadcrumbBar->textField()->isVisible());
+#endif
 
     // Cancel it with the escape key.
-    QTest::keyClick(dialogHelper.window(), Qt::Key_Escape);
+    QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Escape);
     QVERIFY(!breadcrumbBar->textField()->isVisible());
     QVERIFY(dialogHelper.dialog->isVisible());
 
+#if QT_CONFIG(shortcut)
     // Make it visible.
-    QTest::keySequence(dialogHelper.window(), editPathKeySequence);
+    QTest::keySequence(dialogHelper.popupWindow(), editPathKeySequence);
     QVERIFY(breadcrumbBar->textField()->isVisible());
+#endif
 
     // Cancel it with the escape key again.
-    QTest::keyClick(dialogHelper.window(), Qt::Key_Escape);
+    QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Escape);
     QVERIFY(!breadcrumbBar->textField()->isVisible());
     QVERIFY(dialogHelper.dialog->isVisible());
 
     // Pressing escape now should close the dialog.
-    QTest::keyClick(dialogHelper.window(), Qt::Key_Escape);
+    QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Escape);
     QVERIFY(!dialogHelper.dialog->isVisible());
     QTRY_VERIFY(!dialogHelper.quickDialog->isVisible());
 }
@@ -687,6 +679,7 @@ void tst_QQuickFolderDialogImpl::tabFocusNavigation()
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     QList<QQuickItem*> expectedFocusItems;
 
@@ -723,25 +716,25 @@ void tst_QQuickFolderDialogImpl::tabFocusNavigation()
     }
 
     // Tab through each item, checking the focus after each.
-    for (auto expectedFocusItem : qAsConst(expectedFocusItems)) {
+    for (auto expectedFocusItem : std::as_const(expectedFocusItems)) {
         // Check the focus item first so that we account for the first item.
         // Print detailed failure message as workaround for QTBUG-92102.
-        QVERIFY2(dialogHelper.window()->activeFocusItem() == expectedFocusItem, qPrintable(QString::fromLatin1(
-            "\n   Actual:   %1\n   Expected: %2").arg(QDebug::toString(dialogHelper.window()->activeFocusItem()))
+        QVERIFY2(dialogHelper.popupWindow()->activeFocusItem() == expectedFocusItem, qPrintable(QString::fromLatin1(
+            "\n   Actual:   %1\n   Expected: %2").arg(QDebug::toString(dialogHelper.popupWindow()->activeFocusItem()))
                 .arg(QDebug::toString(expectedFocusItem))));
 
         if (expectedFocusItem != expectedFocusItems.last())
-            QTest::keyClick(dialogHelper.window(), Qt::Key_Tab);
+            QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Tab);
     }
 
     // Ensure the order is reversed when shift-tabbing.
     std::reverse(expectedFocusItems.begin(), expectedFocusItems.end());
     // We know the first (last) item has focus already, so skip it.
     expectedFocusItems.removeFirst();
-    for (auto expectedFocusItem : qAsConst(expectedFocusItems)) {
-        QTest::keyClick(dialogHelper.window(), Qt::Key_Tab, Qt::ShiftModifier);
+    for (auto expectedFocusItem : std::as_const(expectedFocusItems)) {
+        QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Tab, Qt::ShiftModifier);
 
-        QCOMPARE(dialogHelper.window()->activeFocusItem(), expectedFocusItem);
+        QCOMPARE(dialogHelper.popupWindow()->activeFocusItem(), expectedFocusItem);
     }
 }
 
@@ -807,13 +800,14 @@ void tst_QQuickFolderDialogImpl::itemsDisabledWhenNecessary()
 {
     // Open the dialog.
     FolderDialogTestHelper dialogHelper(this, "bindCurrentFolder.qml", {},
-        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir2.path()) }});
+        {{ "initialFolder", QUrl::fromLocalFile(tempSubDir2CanonicalPath) }});
     QVERIFY2(dialogHelper.isWindowInitialized(), dialogHelper.failureMessage());
     QVERIFY(dialogHelper.waitForWindowActive());
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
-    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2.path()));
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
+    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempSubDir2CanonicalPath));
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     // We opened it in a folder that has no files, so the Open button should be disabled.
     QVERIFY(dialogHelper.quickDialog->footer());
@@ -828,17 +822,18 @@ void tst_QQuickFolderDialogImpl::itemsDisabledWhenNecessary()
     QVERIFY(breadcrumbBar);
     QVERIFY(clickButton(breadcrumbBar->upButton()));
     QCOMPARE(openButton->isEnabled(), true);
-    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
-    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempDir.path()));
+    COMPARE_URL(dialogHelper.dialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
+    COMPARE_URL(dialogHelper.quickDialog->currentFolder(), QUrl::fromLocalFile(tempDirCanonicalPath));
 
+#if QT_CONFIG(shortcut)
     // Get the text edit visible with Ctrl+L. The Open button should now be disabled.
     const auto editPathKeySequence = QKeySequence(Qt::CTRL | Qt::Key_L);
-    QTest::keySequence(dialogHelper.window(), editPathKeySequence);
+    QTest::keySequence(dialogHelper.popupWindow(), editPathKeySequence);
     QVERIFY(breadcrumbBar->textField()->isVisible());
     QCOMPARE(openButton->isEnabled(), false);
-
+#endif
     // Hide it with the escape key. The Open button should now be enabled.
-    QTest::keyClick(dialogHelper.window(), Qt::Key_Escape);
+    QTest::keyClick(dialogHelper.popupWindow(), Qt::Key_Escape);
     QVERIFY(!breadcrumbBar->textField()->isVisible());
     QCOMPARE(openButton->isEnabled(), true);
 }
