@@ -13,6 +13,7 @@
 #include <QtQuick/QQuickItem>
 #include <QtQuickControls2/qquickstyle.h>
 #include <QQmlApplicationEngine>
+#include <QtQml/qqmlcomponent.h>
 #include <QtQuickTemplates2/private/qquickapplicationwindow_p.h>
 
 #ifdef Q_OS_WIN
@@ -62,7 +63,7 @@ public:
         qCDebug(lcGrabber) << "Using ApplicationWindow as visual parent" << this;
 
         appwindow = qobject_cast<QQuickApplicationWindow *>(window);
-        if (preferAppWindow)
+        if (preferAppWindow && !justShow)
             QObject::connect(appwindow, &QQuickWindow::afterRendering, this, &GrabbingView::startGrabbing);
     }
     QQuickApplicationWindow* appWindow() { return appwindow; }
@@ -219,7 +220,17 @@ int main(int argc, char *argv[])
         {
             // test qml component creation
             QQmlComponent component(engine, QUrl::fromLocalFile(ifile));
+            if (component.isError()) {
+                qWarning().nospace() << "Failed to load QML at " << ifile << ": " << component.errors();
+                return 1;
+            }
+
             auto *itemObject = qobject_cast<QQuickItem*>(component.create());
+            if (component.isError()) {
+                qWarning().nospace() << "Failed to create object instance from QML at "
+                    << ifile << ": " << component.errors();
+                return 1;
+            }
 
             // TODO: Hack to import native style forcefully for windows
             QString appCompStr = "import QtQuick.Controls\n";
