@@ -237,7 +237,7 @@ void QmltcCodeGenerator::generate_createBindingOnProperty(
         }
 
         *block += prologue;
-        *block << u"if (!initializedCache.contains(\"%1\"))"_s.arg(p.propertyName());
+        *block << u"if (!initializedCache.contains(QStringLiteral(\"%1\")))"_s.arg(p.propertyName());
         *block << u"    "_s + value + u"->" + bindable + u"().setBinding(" + createBindingForBindable + u");";
         *block += epilogue;
     } else {
@@ -248,7 +248,7 @@ void QmltcCodeGenerator::generate_createBindingOnProperty(
                 + QString::number(propertyIndex) + u", " + QString::number(valueTypeIndex) + u", "
                 + propName + u")";
         // Note: in this version, the binding is set implicitly
-        *block << u"if (!initializedCache.contains(\"%1\"))"_s.arg(p.propertyName());
+        *block << u"if (!initializedCache.contains(QStringLiteral(\"%1\")))"_s.arg(p.propertyName());
         *block << createBindingForNonBindable + u";";
     }
 }
