@@ -49,7 +49,7 @@ static bool isImplicitComponent(const QQmlJSScope::ConstPtr &type)
         return false;
     const auto cppBase = QQmlJSScope::nonCompositeBaseType(type);
     const bool isComponentBased = (cppBase && cppBase->internalName() == u"QQmlComponent");
-    return type->isComponentRootElement() && !isComponentBased;
+    return type->componentRootStatus() != QQmlJSScope::IsComponentRoot::No && !isComponentBased;
 }
 
 /*! \internal
@@ -74,7 +74,7 @@ QmltcVisitor::QmltcVisitor(const QQmlJSScope::Ptr &target, QQmlJSImporter *impor
                            const QStringList &qmldirFiles)
     : QQmlJSImportVisitor(target, importer, logger, implicitImportDirectory, qmldirFiles)
 {
-    m_qmlTypeNames.append(QFileInfo(logger->fileName()).baseName()); // put document root
+    m_qmlTypeNames.append(QFileInfo(logger->filePath()).baseName()); // put document root
 }
 
 void QmltcVisitor::findCppIncludes()
@@ -318,11 +318,11 @@ void QmltcVisitor::endVisit(QQmlJS::AST::UiProgram *program)
         return;
 
     QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>> bindings;
-    for (const QQmlJSScope::ConstPtr &type : std::as_const(m_qmlTypes)) {
-        if (isOrUnderComponent(type))
-            continue;
+
+    // Yes, we want absolutely all bindings in the document.
+    // Not only the ones immediately assigned to QML types.
+    for (const QQmlJSScope::ConstPtr &type : std::as_const(m_scopesByIrLocation))
         bindings.insert(type, type->ownPropertyBindingsInQmlIROrder());
-    }
 
     postVisitResolve(bindings);
     setupAliases();
@@ -337,6 +337,13 @@ void QmltcVisitor::endVisit(QQmlJS::AST::UiProgram *program)
             checkNamesAndTypes(type);
 }
 
+bool QmltcVisitor::checkCustomParser(const QQmlJSScope::ConstPtr &scope)
+{
+    if (QQmlJSImportVisitor::checkCustomParser(scope))
+        m_seenCustomParsers = true;
+    return false;
+}
+
 QQmlJSScope::ConstPtr fetchType(const QQmlJSMetaPropertyBinding &binding)
 {
     switch (binding.bindingType()) {
@@ -346,73 +353,58 @@ QQmlJSScope::ConstPtr fetchType(const QQmlJSMetaPropertyBinding &binding)
         return binding.interceptorType();
     case QQmlSA::BindingType::ValueSource:
         return binding.valueSourceType();
-    // TODO: AttachedProperty and GroupProperty are not supported yet,
-    // but have to also be acknowledged here
+    case QQmlSA::BindingType::AttachedProperty:
+        return binding.attachedType();
+    case QQmlSA::BindingType::GroupProperty:
+        return binding.groupType();
     default:
         return {};
     }
     Q_UNREACHABLE_RETURN({});
 }
 
+template<typename TypePredicate, typename BindingPredicate>
+void iterateBindings(
+        const QQmlJSScope::ConstPtr &root,
+        const QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>> &qmlIrOrderedBindings,
+        TypePredicate typePredicate, BindingPredicate bindingPredicate)
+{
+    // NB: depth-first-search is used here to mimic various QmlIR passes
+    QStack<QQmlJSScope::ConstPtr> types;
+    types.push(root);
+    while (!types.isEmpty()) {
+        auto current = types.pop();
+
+        if (typePredicate(current))
+            continue;
+
+        Q_ASSERT(qmlIrOrderedBindings.contains(current));
+        const auto &bindings = qmlIrOrderedBindings[current];
+        // reverse the binding order here, because stack processes right-most
+        // child first and we need left-most first
+        for (auto it = bindings.rbegin(); it != bindings.rend(); ++it) {
+            const auto &binding = *it;
+
+            if (bindingPredicate(current, binding))
+                continue;
+
+            if (auto type = fetchType(binding))
+                types.push(type);
+        }
+    }
+}
+
 template<typename Predicate>
 void iterateTypes(
         const QQmlJSScope::ConstPtr &root,
         const QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>> &qmlIrOrderedBindings,
         Predicate predicate)
 {
-    // NB: depth-first-search is used here to mimic various QmlIR passes
-    QStack<QQmlJSScope::ConstPtr> types;
-    types.push(root);
-    while (!types.isEmpty()) {
-        auto current = types.pop();
-
-        if (predicate(current))
-            continue;
-
-        if (isOrUnderComponent(current)) // ignore implicit/explicit components
-            continue;
-
-        Q_ASSERT(qmlIrOrderedBindings.contains(current));
-        const auto &bindings = qmlIrOrderedBindings[current];
-        // reverse the binding order here, because stack processes right-most
-        // child first and we need left-most first
-        for (auto it = bindings.rbegin(); it != bindings.rend(); ++it) {
-            const auto &binding = *it;
-            if (auto type = fetchType(binding))
-                types.push(type);
-        }
-    }
-}
-
-template<typename Predicate>
-void iterateBindings(
-        const QQmlJSScope::ConstPtr &root,
-        const QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>> &qmlIrOrderedBindings,
-        Predicate predicate)
-{
-    // NB: depth-first-search is used here to mimic various QmlIR passes
-    QStack<QQmlJSScope::ConstPtr> types;
-    types.push(root);
-    while (!types.isEmpty()) {
-        auto current = types.pop();
-
-        if (isOrUnderComponent(current)) // ignore implicit/explicit components
-            continue;
-
-        Q_ASSERT(qmlIrOrderedBindings.contains(current));
-        const auto &bindings = qmlIrOrderedBindings[current];
-        // reverse the binding order here, because stack processes right-most
-        // child first and we need left-most first
-        for (auto it = bindings.rbegin(); it != bindings.rend(); ++it) {
-            const auto &binding = *it;
-
-            if (predicate(current, binding))
-                continue;
-
-            if (auto type = fetchType(binding))
-                types.push(type);
-        }
-    }
+    iterateBindings(root, qmlIrOrderedBindings, [predicate](const QQmlJSScope::ConstPtr &current) {
+        return predicate(current) || isOrUnderComponent(current);
+    }, [](const QQmlJSScope::ConstPtr &, const QQmlJSMetaPropertyBinding &) {
+        return false;
+    });
 }
 
 /*! \internal
@@ -460,7 +452,7 @@ void QmltcVisitor::postVisitResolve(
     };
     for (const auto &inlineComponentName : m_inlineComponentNames) {
         iterateBindings(m_inlineComponents[inlineComponentName], qmlIrOrderedBindings,
-                        findDeferred);
+                        isOrUnderComponent, findDeferred);
     }
 
     const auto isOrUnderDeferred = [&deferredTypes](QQmlJSScope::ConstPtr type) {
@@ -588,7 +580,7 @@ void QmltcVisitor::postVisitResolve(
     const auto setRuntimeId = [&](const QQmlJSScope::ConstPtr &type) {
         // any type wrapped in an implicit component shouldn't be processed
         // here. even if it has id, it doesn't need to be set by qmltc
-        if (type->isComponentRootElement()) {
+        if (type->componentRootStatus() != QQmlJSScope::IsComponentRoot::No) {
             return true;
         }
 
