@@ -22,10 +22,10 @@ class QmltcVisitor : public QQmlJSImportVisitor
     void postVisitResolve(const QHash<QQmlJSScope::ConstPtr, QList<QQmlJSMetaPropertyBinding>>
                                   &qmlIrOrderedBindings);
     void setupAliases();
-    void checkForNamingCollisionsWithCpp(const QQmlJSScope::ConstPtr &type);
-    void setRootFilePath();
+    void checkNamesAndTypes(const QQmlJSScope::ConstPtr &type);
+    QString filePath(const QQmlJSScope::ConstPtr &scope) const;
 
-    QString sourceDirectoryPath(const QString &path);
+    QString sourceDirectoryPath(const QString &path) const;
 
     using InlineComponentOrDocumentRootName = QQmlJSScope::InlineComponentOrDocumentRootName;
 
@@ -135,6 +135,9 @@ public:
     enum Mode { Import, Compile };
     void setMode(Mode mode) { m_mode = mode; }
 
+    bool checkCustomParser(const QQmlJSScope::ConstPtr &scope) override;
+    bool hasSeenCustomParsers() const { return m_seenCustomParsers; }
+
 protected:
     QStringList m_qmlTypeNames; // names of QML types arranged as a stack
     QHash<QString, int> m_qmlTypeNameCounts;
@@ -184,6 +187,8 @@ protected:
     QHash<QQmlJSScope::ConstPtr, int> m_typesWithId;
 
     Mode m_mode = Mode::Import;
+
+    bool m_seenCustomParsers = false;
 };
 
 QT_END_NAMESPACE
